#include "bay.h"
#include "clipping.h"
#include "soup.h"
#include "motion.h"
#include "universe.h"
#include "gbahandler.h"
#include "screenmode.h"
#include "gfx.h"
#include "colours.h"
#include "text.h"
#include "flying.h"

#define BAY_DELAY   (60)


typedef struct {
  unsigned char index;
  unsigned char x;
  unsigned short z;
} bay_pos_t;

// LDA$34, LSRA, then RORA to x
// sign is bit 0 of z
static const bay_pos_t bay_pos[] = {
  {0x02, 0x54, 0x3b}, 
  {0x03, 0x82, 0xb0}, 
  {0x00, 0x00, 0x00},
  {0x01, 0x50, 0x11},
  {0x01, 0xd1, 0x28}, 
  {0x01, 0x40, 0x06},
  {0x03, 0x60, 0x90}, 
  {0x04, 0x10, 0xd1}, 
  {0x00, 0x00, 0x00},
  {0x06, 0x51, 0xf8},
  {0x07, 0x60, 0x75},
  {0x00, 0x00, 0x00},
  
};

/*
void add_ship2(void) {
  ships = ship_collections[13];
  int ship_index = 3;
  int ship_id = add_new_ship(ship_index);
  stellarobject_t * obj = &g_objects[ship_id]; 
  obj->location.z = 0x100;
  obj->location.x = 0;
  obj->location.y = 0;
  entry(obj, ships[ship_index-1]);
  
}
*/

static void add_ship(int z, int x, int index) {
  // hang on, might be a null entry
  if (!index) {
    return;
  }
  int ship_index = index+SHIP_SST;
  while (index) {
    ship_index++;
    if (ships[ship_index-1])
      index--;
  }
  int ship_id = add_new_ship(ship_index);
  // 1FA8 - adds ship and draws it
  stellarobject_t * obj = &g_objects[ship_id]; 
  // x is negative if z has bit 0 set
  obj->location.x = (z&0x1)?-x:x;
  // z is 256 further away if x has bit 0 set
  obj->location.z = z | ((x&1)?0x100:0x200);
  
  // rotate the ship around its y axis a random amount
  // each rotation is ~3 degrees
  int r = get_rand();
  int i;
  for (i = 0; i < r; i++) {
    fixed_rotate(&obj->rotmat[0].x,&obj->rotmat[2].x,r);
    fixed_rotate(&obj->rotmat[0].y,&obj->rotmat[2].y,r);
    fixed_rotate(&obj->rotmat[0].z,&obj->rotmat[2].z,r);
  }

  // y pos is  sqr(target_size)-0x64
  int size = bios_sqrt(ships[ship_index-1]->target_size);

  obj->location.y = -( ((100 - size)/2)&0xff ); 
  tidy(obj->rotmat);
  entry(obj, ships[ship_index-1]);
}

void kwix_line_vert(int x) {  
  // draw a line until we collide with the ship
  int i;
  for (i = 0; i < CLIP_Y_MAX ; i++) {
    int pix = get_pixel(x, i);
    if (pix) {
      break;
    }
    put_pixel(x,i);
  }
}

void kwix_line_horiz(int y) {

  // draw a line until we collide with the ship
  int i;
  for (i = 0; i < CLIP_X_MAX ; i++) {
    int pix = get_pixel(i, y);
    if (pix) {
      break;
    }
    put_pixel(i, y);
  }
  for (i = (CLIP_X_MAX-1); i > 0 ; i--) {
    int pix = get_pixel(i, y);
    if (pix) {
      break;
    }
    put_pixel(i, y);
  }
}

void show_bay(void) {
  // 1EEC - decide which ships to show
  // the collection is not random - it is based on the docked code
  // which only has 1 table of ships ...
#ifdef HAVE_GBA_SHIPSETS
  extern const shipheader_t * const ship_bay_collection[31];
  {
    int i;
    for (i = 0; i < 31; i++) {
      ships[i] = ship_bay_collection[i];
    }
  }
#else
  ships = ship_collections[15];
#endif
  swap_buffer();
  clear_screen();
  clear_text();
  update_dials();
  int r = get_rand();
  if (r&0x80) {
    //1EF9
    int posind = (r&3)*3;
    add_ship(bay_pos[posind].z,bay_pos[posind].x,bay_pos[posind].index);
    posind++;
    add_ship(bay_pos[posind].z,bay_pos[posind].x,bay_pos[posind].index);
    posind++;
    add_ship(bay_pos[posind].z,bay_pos[posind].x,bay_pos[posind].index);
  } else {
    //1F22
    int x = r/2;
    int z = get_rand();
    int i = get_rand()&7;
    add_ship(z,x,i);
  }
  lineColour = VECTOR_COL;

  int i;
  for (i = 0; i < 8; i++) 
    kwix_line_horiz(SCREEN_SIZE_Y + i*i);
  for (i = 0; i < 15; i++) 
    kwix_line_vert(i*16);
  
  swap_buffer();
  for (i = 0; i < BAY_DELAY; i++)
    wait_vsync_int();
}

