
#include "gfx.h"
#include "text.h"
#include "dialgauge.h"
#include "sprite.h"
#include "commander.h"
#include "eliteloop.h"

#define GAUGE_TILE   640
//(FONT_TILE+FONT_GLYPHS)

void setup_dial_gauges(void) {
  /*
   * Dial gauges (altitude, etc) are in the text bg
   * */

  // 0, f<<28, f<<24, f<<20, f<<16, f<<12, f<<8, f<<4, f
  // creates a masked series of graphics

  int i;
  u16 dial_mask_low  = 0x0000;
  u16 dial_mask_high = 0x0000;

  int shiftl = 16;
  int shifth = 16;

  u32 * pixels = (u32*)&bg_text.tileData[(GAUGE_TILE)*8*2];
  while (shiftl>=0) {
    
    *pixels++ = 0;
    for (i = 0; i < 4; i++) {
      *pixels = (dial_mask_low)|(dial_mask_high<<16);
      pixels++;
    }
    *pixels++ = 0;
    *pixels++ = 0;
    *pixels++ = 0;
    
    shifth-=4;
    if (shifth >= 0) {
      dial_mask_low |= (FONT_COL_INDEX)<<(12-shifth);
    } else {
      shiftl-=4;
      dial_mask_high |= (FONT_COL_INDEX)<<(12-shiftl);
    }
  }
}

typedef struct {
  u8 x;
  u8 y;
  u8 col0;  // 0-25%
  u8 col1;  // 25-75%
  u8 col2;  // 75+%
} dialdata_t;

#define DIAL_X_0     2
#define DIAL_X_1     25
#define DIAL_Y_OFFSET  (160/8 - 6)

// missing altitude!
const dialdata_t dial_positions[] = {
  {DIAL_X_0,0,TEXT_PAL_RED,TEXT_PAL_PURPLE, TEXT_PAL_PURPLE},  // FS
  {DIAL_X_0,1,TEXT_PAL_RED,TEXT_PAL_PURPLE, TEXT_PAL_PURPLE},  // AS
  {DIAL_X_0,2,TEXT_PAL_WHITE, TEXT_PAL_RED, TEXT_PAL_RED},        // EN

  {DIAL_X_0,3,TEXT_PAL_BLUE, TEXT_PAL_BLUE, TEXT_PAL_BLUE},      // FU
  {DIAL_X_0,4,TEXT_PAL_ORANGE, TEXT_PAL_ORANGE, TEXT_PAL_RED},     // CT  \ one or
  {DIAL_X_0,4,TEXT_PAL_RED, TEXT_PAL_GREEN, TEXT_PAL_GREEN},    // AL  /    the other
  {DIAL_X_0,5,TEXT_PAL_ORANGE, TEXT_PAL_ORANGE, TEXT_PAL_RED},  // LT

  {DIAL_X_1,3,TEXT_PAL_ORANGE, TEXT_PAL_ORANGE, TEXT_PAL_RED},     // SP
};



// for example energy uses 4 dials on elite
// here uses just 1
// in 6502, energy/4 results in 64 pixels
// split into 4 dials of 16 pixels each
// in this version, the dials are 32 pixels each
// so energy would be split into energy/8
// same for shields - shields/8
// Fuel in elite is fuel/4 - here can be fuel/2
void dial(int index, int value)  {
  // do not draw if we are not on the "main" screen
  if (g_my_stats.screen != SCR_LAUNCHED)
    return;
  
  int pal;
  if (value >= 24) {
    // colour top 25%
    pal = dial_positions[index].col2;
    if (value > 32)
      value = 32;
  } else if (value < 8) {
    // low 25% colour
    pal = dial_positions[index].col0;
  } else {
    // std colour
    pal = dial_positions[index].col1;
  }
  
  int map_index =  (dial_positions[index].x) + 
    (dial_positions[index].y+DIAL_Y_OFFSET)*32;

  int i = 0; 
  while (value > 8) {
    bg_text.mapData[map_index+i] = (GAUGE_TILE+8 )|PALETTE(pal);
    i++;
    value-=8;
  }
  if (value > 0) {
    bg_text.mapData[map_index+i] = (GAUGE_TILE + value)|PALETTE(pal);
    i++;
  } 
  while (i < 4) {
    bg_text.mapData[map_index+i] = FONT_TILE;
    i++;
  }
}

void showroom_dial(int value, int max, int x, int y, int pal, int pal_off) 
{
  int map_index =  x + y*32;
  int i = 0; 
  while (value) {
    bg_text.mapData[map_index+i] = (GAUGE_TILE+7 )|PALETTE(pal);
    i++;
    value--;
  }
  while (i < max) {
    bg_text.mapData[map_index+i] = (GAUGE_TILE+7 )|PALETTE(pal_off);
    i++;
    value--;
  }
  
}

