#include "docked.h"
#include "sprite.h"
#include "gfx.h"
#include "text.h"
#include "status.h"
#include "eliteloop.h"
#include "chart.h"
#include "navigation.h"
#include "commander.h"
#include "upgrade.h"
#include "market.h"
#include "launch.h"
#include "find.h"
#include "encyclo.h"
#include "soup.h"
#include "loadsave.h"
#include "options.h"
#include "input.h"
#include "flying.h"
#include "missions.h"
#include "soundfx.h"

#include "sxmp.h"

#define DOCKED_MENU_REPEAT  30


void stay_docked(void) {
  if (g_commander.cash >= 500) {
    g_commander.cash -= 500;
    deliver_special_cargo();
    g_commander.planet_random = get_rand();
    calculate_market();
    manual_activate_screen(SCR_STATUS+1);
  }
}


/*
In station:

"station"
Launch
Pay Docking Fee to Stay 

"cargo"
Buy Cargo  (Market Prices)
Buy Special Cargo
Sell Cargo


"ship"
Sell Ship Equipment
Buy Ship Equipment
Buy New Ship

"chart"
Galactic Chart
Short Chart
Data on planet

Market prices  (obsolete?)

"commander"
Status
Inventory

Access Encyclopedia Galactica


In space

"Primary" things
Target Missile
Fire Missile
Activate Hyperspace Unit
Fire Escape pod
Micro Jump
Activate ECM
Front View
Rear View
Left View
Right View

"Secondary" features:
Unarm Missile
Hyperspace Jump
Galactic Hyperspace Jump
Toggle compass Sun/Panet
Docking Computer on/off

Plus:
Galactic Chart
Short Chart
Data on planet
Market prices
Status
Inventory

*/

int g_panel_item = 0;
int g_panel_timer = 0;

typedef struct {
  FunctionPtr function;
  char * name;
} docked_function_t;

const docked_function_t g_docked_functions[SELECT_COUNT+2] = {
  {launch,"Launch"},
  {stay_docked,"Remain Docked"},
  {show_market_prices,"Buy/Sell Cargo"},
  {show_special_cargo,"Buy Special Cargo"},
  {equip_ship,"Equip Ship"},
  {sell_ship_equipment,"Sell Ship Equipment"},
  {buy_ship,"Buy Ship"},
  {long_scrn,"Galactic Map"},
  {short_scrn,"Short Range Map"},
  {find_planet,"Find System"},
  {show_current_planet,"Data On Planet"},
  {show_status,"Status Page"},
  {show_inventory,"Inventory"},
  {equipment_screen,"View Equipment"},
  {encyclo,"Encyclopedia"}, 
  {game_options,"options"},
  {save_game,"save"},
};

void show_docked_panel(void) {
  // the docked panel is a sprite based panel
  // show the sprites
  g_panel_timer = PANEL_TICKS;
  activate_sprites(GIZMO_SPRITE, GIZMO_SPRITE+SELECT_COUNT);
  select_sprite(g_panel_item);
  fade_bg_sprites(FADE_LIMIT);
  sprite_text(g_docked_functions[g_panel_item].name);
}
void hide_docked_panel(void) {
  // the docked panel is a sprite based panel
  // hide the sprites
  g_panel_timer = 0;
  g_galactic_hyper_preliminary = 0;
  deselect_sprite(g_panel_item);
  deactivate_sprites(GIZMO_SPRITE, GIZMO_SPRITE+SELECT_COUNT);
  deactivate_sprites(SPRITE_TEXT_SPRITE, SPRITE_TEXT_SPRITE+8);
}
void start_hide_docked_panel(void) 
{
  if (g_panel_timer > FADE_LIMIT)
    g_panel_timer = FADE_LIMIT;
  deactivate_sprites(SPRITE_TEXT_SPRITE, SPRITE_TEXT_SPRITE+8);
}

void docked_panel_decrsel(void) {
  if (g_panel_timer <= 0) {
    show_docked_panel();
    return;
  } else if (g_panel_timer < FADE_LIMIT) {
    // make fully visible
    //fade_bg_sprites(FADE_LIMIT);
    show_docked_panel();
  }
  g_panel_timer = PANEL_TICKS;
  deselect_sprite(g_panel_item);
  g_panel_item--;
  if (g_panel_item < 0) {
    g_panel_item = SELECT_COUNT-1;
  }
  sprite_text(g_docked_functions[g_panel_item].name);
  
  //play_sound(SOUNDFX_MENU_CLICK);
  select_sprite(g_panel_item);
}

void docked_panel_incrsel(void) {
  if (g_panel_timer <= 0) {
    show_docked_panel();
    return;
  } else if (g_panel_timer < FADE_LIMIT) {
    // make fully visible
    //fade_bg_sprites(FADE_LIMIT);
    show_docked_panel();
  }

  g_panel_timer = PANEL_TICKS;
  deselect_sprite(g_panel_item);
  g_panel_item++;
  if (g_panel_item == SELECT_COUNT) {
    g_panel_item = 0;
  }

  sprite_text(g_docked_functions[g_panel_item].name);
  //play_sound(SOUNDFX_MENU_CLICK);
  
  select_sprite(g_panel_item);
}

void activate_selection(void) {
  if (g_docked_functions[g_panel_item].function != 0) {
    if (g_my_stats.screen == SCR_ENCYCLO)
      reset_icon_sprites();
    set_bay_bgs();
    draw_border();
    if (g_my_stats.docked) {
      g_timer_fn = common_panel_timer;
    }
    //play_sound(SOUNDFX_MENU_ACCEPT);
    g_key_handler = common_docked_keys;
    start_hide_docked_panel();
    g_docked_functions[g_panel_item].function();
  }
}

void manual_activate_screen(int screen) {
  int initial_pi = g_panel_item;
  g_panel_item = screen-1;
  activate_selection();
  if (g_panel_item >= SELECT_COUNT)
    g_panel_item = initial_pi;
}


void bay(void) {
  sxmp_fade_out();
  g_my_stats.docked = 1;

  set_bay_bgs();
  draw_border();
  
  deliver_special_cargo();

  // after special cargo check, update our mission status
  // $1226
  check_mission();
  
  show_status();
  set_row_pal(0,TEXT_PAL_YELLOW);
  g_panel_item = SCR_STATUS;

#if 0
  // for debugging, saves faffing about with menus
  show_docked_panel();
  launch();
#endif
}

// the common keys are used for the L+R panel
void common_docked_keys(void) {
  // use repeating to stop keys bouncing crazily
  // when pressed once, the key is on
  // if repeated several times, the key is on
  user_menu_controls(DOCKED_MENU_REPEAT);
  if (g_virtual_keys[EVK_MENU_R]) {
    docked_panel_incrsel();
  }
  if (g_virtual_keys[EVK_MENU_L]) {
    docked_panel_decrsel();
  }
  if (g_virtual_keys[EVK_MENU_A]) {
    activate_selection();
  }
  if (g_virtual_keys[EVK_MENU_B]) {
    start_hide_docked_panel();
  }
  if (g_virtual_keys[EVK_MENU_START]) {
    if (g_my_stats.screen == SCR_OPTIONS ||
        g_my_stats.screen == SCR_SAVE_SCREEN
       ) 
    {
      return;
    }
    if (g_my_stats.screen == SCR_ENCYCLO)
      reset_icon_sprites();
    hide_docked_panel();
    game_options();
  }
}

void common_panel_timer(void) {
  animate_selection();
  if (g_panel_timer)
    g_panel_timer--;
  if (g_panel_timer < FADE_LIMIT && g_panel_timer > 1) {
    // fade between bg and panel
    fade_bg_sprites(g_panel_timer);
  }
  if (g_panel_timer == 1) {
    hide_docked_panel();
  }
  else {
    if ((g_panel_timer&15) == 0) {
      if (g_my_stats.docked == 0) {
        // this magically refreshes on certain events...
        reload_inflight_panel();
        if ( g_galactic_hyper_preliminary)
          g_galactic_hyper_preliminary--;
      }
    }
  }
}

