/* 
 * The encyclopedia code makes use of some of the same
 * tricks as the Goat Soup generator. The custom
 * dictionaries have been taken directly from 
 * memory dumps of the game under emulation.
 *
 *
 * */

#include "gbahandler.h"
#include "text.h"
#include "soup.h"
#include "encyclo.h"
#include "eliteloop.h"
#include "commander.h"
#include "docked.h"
#include "menu.h"
#include "shipdata.h"
#include "universe.h"
#include "gfx.h"
#include "options.h"



typedef struct {
  u8 x;
  u8 y;
  u8 t;

} datasheet_t;

// this point to the tables of data
extern const unsigned char * const * const datasheet_tables[];

// each ship uses these to find the header entries...
const unsigned char header_table_index[] = {
  0,  // 1 Inservice Date
  2,  // 2 Combat Factor
  6,  // 3 Dimensions
  8,  // 4 Speed
  10, // 5 Crew
  12, // 6 Range
  14, // 7 Cargo Space
  17, // 8 Armaments
  19, // 9 Hull
  21, // 10 Drive Motors
  23, // 11 Space 

};
const datasheet_t header_table[] = {
  {0x01,0x02,0x25},  // Inservice Date:
  {0x01,0x03,0x00},
  
  {0x18,0x06,0x26},  // Combat
  {0x18,0x07,0x2f},  // Factor:
  {0x18,0x08,0x41},  // CF
  {0x1a,0x08,0x00},  
  
  {0x01,0x05,0x2b},  // Dimensions:
  {0x01,0x06,0x00},
  
  {0x18,0x03,0x24},  // Speed:
  {0x18,0x04,0x00},
  
  {0x18,0x0a,0x27},  // Crew:
  {0x18,0x0b,0x00},
  
  {0x18,0x0d,0x29},  // Range:
  {0x18,0x0e,0x00},
  
  {0x01,0x08,0x3d},  // Cargo
  {0x01,0x09,0x2d},  // Space:
  {0x01,0x0a,0x00},
  
  {0x01,0x10,0x23},  // Armaments:
  {0x01,0x11,0x00},
  
  {0x17,0x10,0x2c},  // Hull:
  {0x17,0x11,0x00},
  
  {0x01,0x0c,0x28},  // Drive Motors:
  {0x01,0x0d,0x00},
  
  {0x01,0x14,0x2d},  // Space:
  {0x01,0x15,0x00}
};

// index of the shipset that contains the ship data
#ifdef HAVE_GBA_SHIPSETS
const u8 shipset_ids[] = {
0x0b,0x0d,0x17,0x0A,0x13,0x16,0x0C,0x18,
0x01,0x01,0x02,0x0C,0x11,0x0A,0x13,0x10,
0x10,0x0A,0x15,0x14,0x0A,0x08,0x0E,0x1C,
0x1D,0x09,0x0F,0x0E, /*salamanderhack*/0x0B
};
#else
const u8 shipset_ids[] = {
0x00,0x07,0x08,0x0A,0x09,0x0F,0x01,0x0D,
0x00,0x01,0x00,0x0C,0x04,0x01,0x06,0x08,
0x0C,0x00,0x0E,0x05,0x04,0x0B,0x0B,0x02,
0x02,0x0F,0x00,0x07, /*salamanderhack*/2
};
#endif
// x position of the ship title
const u8 ship_title_x[] = {
0x0d,0x0c,0x0c,0x0b,0x0d,0x0c,0x0b,0x0b,
0x08,0x07,0x09,0x0a,0x0d,0x0c,0x0d,0x0d,
0x0d,0x0c,0x0d,0x0c,0x0d,0x0c,0x0b,0x0c,
0x0c,0x0a,0x0d,0x0e, /*salamanderhack*/7
};

// index of the ship within its shipset 
#ifdef HAVE_GBA_SHIPSETS
const u8 index_to_type[] = {
0x00,0x01,0x02,0x03,0x01,0x00,0x00,0x00,
0x00,0x01,0x00,0x04,0x00,0x01,0x00,0x04,
0x03,0x00,0x00,0x00,0x02,0x01,0x00,0x00,
0x00,0x00,0x00,0x01, /*salamanderhack*/0x02
};
#else
const u8 index_to_type[] = {
0x13,0x0e,0x1b,0x0b,0x14,0x0c,0x11,0x0b,
0x02,0x02,0x03,0x19,0x11,0x0b,0x14,0x11,
0x11,0x0b,0x16,0x15,0x0b,0x09,0x11,0x1d,
0x1e,0x0a,0x10,0x0f, /*salamanderhack*/12
};
#endif

// distance (*256) the ship is held from the camera
const u8 min_distances[] = {
0x01,0x02,0x01,0x02,0x01,0x01,0x01,0x02,
0x04,0x04,0x01,0x01,0x01,0x02,0x01,0x02,
0x01,0x02,0x01,0x01,0x02,0x01,0x01,0x03,
0x01,0x01,0x01,0x01,  /*salamanderhack*/1
};

int g_shipsetid = 0;
// used to animate the ship "asynchronously"
void encyc_ship_anim(void) {
  common_panel_timer();
  if (game_frames <  g_game_options.frame_rate)
    return;
  game_frames = 0;
  int i = 0;
  const shipheader_t * const data = ships[g_objects[i].type-1];
  if (data) {
    if( g_objects[i].location.z/256 != (min_distances[g_shipsetid])) {
      g_objects[i].location.z -= 0x100;
    }
    g_objects[i].location.z &= ~0xff;
    g_objects[i].location.z |= 0x20;
    if ( g_my_stats.screen == SCR_ENCYCLO_SHIP) {
      g_objects[i].location.x = 0;
      g_objects[i].location.y = 0;
    } else {
      g_objects[i].location.x = 0;
      g_objects[i].location.y = 0;
    }
    swap_buffer();
    clear_screen();
    mveit(&g_objects[i], data, i);
    entry(&g_objects[i], data);
    draw_border();
  }
}

void setup_encyclo_ship(int shipsetid) {
  // "load" the ship set...
  g_shipsetid = shipsetid;
  set_text_x(ship_title_x[shipsetid]);
  g_yc = 0;

  // title...
  set_soup_caps(1);
  g_text_palette = TEXT_PAL_YELLOW;
  encyclo2buffer(7 + shipsetid);
  g_text_palette = TEXT_PAL_WHITE;
  set_soup_caps(0);
  zero_objects();
#ifdef HAVE_GBA_SHIPSETS
  // has to be different for the gba as we can access all ships at once
  ships[SHIP_WORM-1] = ship_collections[ shipset_ids[shipsetid]  ]->ships[index_to_type[shipsetid]];
  int i = add_new_ship(SHIP_WORM);
#else
  ships = ship_collections[ shipset_ids[shipsetid]  ];
  int i = add_new_ship(index_to_type[shipsetid]);
#endif
  g_objects[i].rotx = 0x7f;
  g_objects[i].rotz = 0x7f;
  g_objects[i].location.z = 0x4000;
  g_objects[i].location.x = 0;
  g_objects[i].location.y = 0;

  g_timer_fn    = encyc_ship_anim;
}

//int g_current_ship_id=-1;
void ship_info(int shipid, int page) {
  // 
  enable_menu(0);
  g_my_stats.screen = SCR_ENCYCLO_SHIP;
  g_shipsetid = shipid;
  if (page) {
    g_shipsetid += 0xe;
  }
  setup_encyclo_ship(g_shipsetid); 

  // ship text from 3554
  const unsigned char * const * dsp = datasheet_tables[g_shipsetid];
  // loop over all the strings in *dsp

  int hindex = 0;
  for (;;) {
    const unsigned char * str =  dsp[hindex++];
    int headerid = (*str);
    if (!headerid) {
      // emptry string
      break;
    }
    headerid--;
    const datasheet_t * ds = &header_table[header_table_index[headerid]];
    // print out the headers while ds->t != 0 
    while (ds->t != 0) {
      set_text_x(ds->x);
      g_yc = ds->y;
      set_soup_caps(1);
      g_text_palette = TEXT_PAL_GREEN;
      encyclo2buffer(ds->t);
      g_text_palette = TEXT_PAL_WHITE;
      ds++;
    }
    set_text_x(ds->x);
    g_yc = ds->y;
    set_soup_caps(0);
    // now the rest of this is taken up by *dsp bytes
    str++;
    while (*str) {
      unsigned char c = *str++;
      if ( c & 0x80) {
        // negative, check d7-ness
        if (c < 0xd7) {
          //set_soup_caps(0);
          encyclo2buffer(c&0x7f);
        } else {
          // subtract d7 and get QQ16 pair
          process_letter(c);
        }
      } else {
        if (c < 0x20) {
          process_letter(c);
        } else {
          // write the letter directly
          handle_soup_letter(c);
          if (c == ' ') {
            set_soup_caps(0);
          }
        }
      }
    }
  }

}

// 369E
static const unsigned char option_count[] = {
  0x04,0x0e,/*salamanderhack 0xf*/0x0e,0x08,0x0d
};
//3699
static const unsigned char option_offset[] = {
  0x02,0x07,0x15,121,0x5f
};
// 368F
static const unsigned char title_string[] = {
  0x01,0x02,0x03,0x05,0x04
};
// 3694
static const unsigned char x_pos[] = {
  0x06,0x0c,0x0c,0x0c,0x0b
};
// 36A3  not sure of this one
static const unsigned char y_pos[] = {
  0x06,0x43,0x43,0x05,0x04
};

static const unsigned char page_index[] = {
  0, 1, 2, 4, 3
};

int encyc_main_accept(int index);
int encyc_ship1_accept(int index);
int encyc_ship2_accept(int index);
int encyc_equip_accept(int index);
int encyc_control_accept(int index);
int encyc_ship2_accept(int index);
int encyc_gen_cancel(int index);

static const MenuFunctionPtr encyc_accept[] = {
  encyc_main_accept,
  encyc_ship1_accept,
  encyc_ship2_accept,
  encyc_equip_accept,
  encyc_control_accept,
  0,
};
static const MenuFunctionPtr encyc_cancel[] = {
  0,
  encyc_gen_cancel,
  encyc_gen_cancel,
  encyc_gen_cancel,
  encyc_gen_cancel,
  0,
};



void draw_choice(int page) {
  // 3623
  g_my_stats.screen = SCR_ENCYCLO;
  clear_text();
  draw_border();
  textmod = ALL_CAPS;
  set_soup_caps(1);
  set_text_x(x_pos[page]);
  g_yc = 0;
  g_text_palette = TEXT_PAL_YELLOW;
  encyclo2buffer(title_string[page]);
  g_text_palette = TEXT_PAL_WHITE;
  g_yc++;

  int i;
  for (i = 0; i < option_count[page]; i++) {
    handle_soup_letter(LF);
    set_soup_caps(0);
    encyclo2buffer(i+option_offset[page]);
  }
}

int encyc_main_accept(int index) {
  int page = page_index[index+1];
  draw_choice(page);
  reset_menu(option_count[page],2);
  set_menu_callbacks(0,0,encyc_accept[index+1], encyc_cancel[index+1]);
  return 0;
}

int encyc_ship1_cancel(int index) {
  //clear_screen();
  set_bay_bgs(); 
  enable_menu(1);
  encyc_main_accept(0);
  set_menu_selection(index);
  return 0;
}
int encyc_ship2_cancel(int index) {
  //clear_screen();
  set_bay_bgs(); 
  enable_menu(1);
  encyc_main_accept(1);
  set_menu_selection(index);
  return 0;
}

int encyc_ship1_accept(int index) {
  // draw the info on ship from sheet 1
  clear_text();
  ship_info(index, 0);
  set_menu_callbacks(0,0,encyc_ship1_cancel, encyc_ship1_cancel);
  return 0;
}

int encyc_ship2_accept(int index){
  // draw the info on ship from sheet 2
  clear_text();
  ship_info(index, 1);
  set_menu_callbacks(0,0,encyc_ship2_cancel, encyc_ship2_cancel);
  return 0;
}

int encyc_equip_cancel(int index) {
  enable_menu(1);
  encyc_main_accept(2);
  set_menu_selection(index);
  return 0;
}
int encyc_equip_accept(int index) {
  // accept on the equip list, draw details of equipment
  // any button returns to equipment list
   
  clear_text();
  g_yc = 0;
  set_text_x(11);
  g_text_palette = TEXT_PAL_YELLOW;
  encyclo2buffer(0x5f+index);
  g_yc++;
  handle_soup_letter(LF);
  set_text_x(1);
  g_text_palette = TEXT_PAL_WHITE;
  encyclo2buffer(0x6c+index);
  set_menu_callbacks(0,0,encyc_equip_cancel, encyc_equip_cancel);
  enable_menu(0);
  return 0;
}

int encyc_control_cancel(int index) {
  reset_icon_sprites();
  enable_menu(1);
  encyc_main_accept(3);
  set_menu_selection(index);
  return 0;
}
int encyc_control_accept(int index) {
  clear_text();
  g_yc = 0;
  set_text_x(11);
  reset_icon_sprites();
  g_text_palette = TEXT_PAL_YELLOW;
  // title
  encyclo2buffer(121+index);
  handle_soup_letter(LF);
  set_text_x(1);
  g_text_palette = TEXT_PAL_WHITE;
  encyclo2buffer(0x57+index);
  // replace encyclo2buffer with a special routine to draw the page chosen
  
  set_menu_callbacks(0,0,encyc_control_cancel, encyc_control_cancel);
  enable_menu(0);
  return 0;
}


int encyc_gen_cancel(int index){
  draw_choice(page_index[0]);
  reset_menu(option_count[0], 2);
  set_menu_callbacks(0,0,encyc_accept[0], encyc_cancel[0]);
  return 0;
}

// galactic encyclopedia
void encyclo(void) {

  // encyclopedia code. in disc version, this is loaded
  // in when ctrl f6 is pressed. 
  // code eventually jumps to 3439, where the initial list
  // of choices is drawn... that is this code here:
  g_my_stats.screen = SCR_ENCYCLO;
  g_key_handler = common_docked_keys;
  g_timer_fn    = common_panel_timer;
  set_text_x(6);
  g_yc = 0;

  draw_choice(page_index[0]);
  reset_menu(option_count[0], 2);
  set_menu_callbacks(0,0,encyc_accept[0], encyc_cancel[0]);
  
}

