
#include <stdlib.h>
#include "gba.h"
#include "sleep.h"
#include "timers.h"
#include "gfx.h"
#include "screenmode.h"
#include "interrupts.h"
#include "sprite.h"
#include "gbahandler.h"
#include "universe.h"
#include "soundfx.h"
#include "commander.h"

#include "sxmp.h"

int game_frames = 0;

void intr_main();

void vblank_ir(void) {
  sxmp_vsync();
  game_frames++;
#if 0
  if (g_my_stats.slow_laser)
    g_my_stats.slow_laser--;
#endif
}
void key_ir(void) {
  Sleep_Mode(0);
}

void (*IntrTable[])() = {
  vblank_ir, // v-blank
  0, // h-blank
  sxmp_update, // vcount
  0, // timer0
  0, // timer1
  sound_ir, // timer2
  0, // timer3
  0, // serial
  0, // dma0
  0, // dma1
  0, // dma2
  0, // dma3
  key_ir, // key
  0  // cart
};

/* Mode 0 
 * BG0   main view  swap "buffers" to draw lines
 *       in reality swap the tile map. The drawable region
 *       is 240*112 = 30*14 = 420 tiles
 *       buffer0 - tiles 0-420
 *       
 * BG1
 *
 */ 

int get_gba_checksum(void) ;
int detect_gba(void) LONG_CALL;
int g_is_gba = 1;
void initialise_gba(void) {
  screen_white(16);
  SetMode(MODE_0 | OBJ_ENABLE | OBJ_MAP_1D);
  init_interrupts(INT_VBLANK|INT_VCOUNT|INT_TIMER2, intr_main);

  g_is_gba = detect_gba();
  if (g_is_gba) {
    g_is_gba = get_gba_checksum();
    if (g_is_gba == 0xBAAE1880) // DS
      g_is_gba = 2;
    else if (g_is_gba == 0xBAAE187F) // GBA
      g_is_gba = 1;
    else // emu ?
      g_is_gba = 0;
  }
  
  screen_white_in();
  screen_fade_out();

  init_sound(); 
  init_universe();
  init_bgs();
  sxmp_init(); 

  // control timer interrupt
  SetTimerData(2,0xf200);
  SetTimerControl(2,TIME_FREQUENCY_64 | TIME_ENABLE | TIME_IRQ_ENABLE);
  // enable VBLANK and VCOUNT interrupts
  REG_DISPSTAT = 8|32;
}


