
#ifndef commander_h_seen
#define commander_h_seen
#include "navigation.h"
#include "market.h"

#define ECM_LENGTH            0x20

#define FRONT_LASER           0
#define REAR_LASER            1
#define RIGHT_LASER           2
#define LEFT_LASER            3
#define IFF_SYSTEM            4
#define ECM_SYSTEM            5
#define FUEL_SCOOPS           6
#define ESCAPE_POD            7
#define HYPERSPACE_UNIT       8
#define ENERGY_UNIT           9
#define DOCKING_COMPUTERS     10
#define GALACTIC_HYPERSPACE   11
#define MISSILES              12

#define EQUIPMENT_LIST        13


typedef struct {

  char name[8]; // 1181  0xd terminated in 6502 
                // (some bugs in the name address?)

  unsigned int cash;         // 0361,362,363,364 
  unsigned char fuel;        // 0365  fuel present * 10
  unsigned char galaxy;      // 0367  GCNT?

  //unsigned char laser[4];    // 0368, 369, 36A, 36B laser strengths on 4 sides
  
  unsigned char ship;        // 036D  current ship type
  unsigned char mission;     // 0358  current mission bit flag

  // equipment fitted
  /*
  char IFF;                  //036E
  char ecm;                  //0380
  char fuel_scoops;          //0381
  char hyperspace_unit;      //0382
  char energy_unit;          //0383
  char docking_computers;    //0384
  char galactic_hyperspace;  //0385
  */
  unsigned char equipment[EQUIPMENT_LIST];  // 4 lasers, rest of equipment
  galcoord_t cpl_coord;
  seed_t gal_seed;

  //unsigned char missiles;       // 038B
  unsigned char legal_status;   // 038C
  unsigned char planet_random;  // 039E  calculated when we first enter the system
                                //       OR when we stay overnight
                                // used to calculate stock market and special cargos     

  unsigned int  kills;         // 039F 03A0 used for rating (harmless, etc)

  unsigned short cargo_space;   // 03D9 space left after fitting equipment
  unsigned char cargo[17]; // 36F - 37E  current ship cargo
  unsigned char station_cargo[17];  // 038D-039B current amount in station

  galcoord_t  special_coord;  // special cargo coords 389 38A
  unsigned short reward;      // special cargo reward 388 387


  unsigned short trumbles;    // 04C9 04CA in C64 Elite

} commander_t;

typedef struct {
  unsigned int  hour;
  unsigned char minute;
  unsigned char second;
  unsigned char subsec;
  unsigned char pad;
} gametime_t;

typedef struct {
  unsigned char  docked;           // not needed, as code is changed for docked/space
                                   // with new routines loaded from disc (BBC)
  unsigned char  game_over;        // BBC uses a big JMP to do the game over

  unsigned char  front_shield;     // 03A5
  unsigned char  aft_shield;       // 03A6
  unsigned char  energy;           // 03A7

  unsigned char  laser_temp;       // 0347
  unsigned char  laser_strength;       // 0343
  unsigned char  laser_on;         // $44
  signed   short    slow_laser;       // 0346
  signed char    ships_present;    // 0349
  unsigned char  ecm_counter;              // $30
  unsigned char  player_ecm;              // $30

  signed char  missile_armed;              // $0344 (armed) and $45 (target)
  
  unsigned char  view;             // 0345
  unsigned char  auto_pilot;       // 033F is the docking computer on?

  unsigned char  space_junk;        // 033E  how much junk is in the zone
  unsigned char  miss_jump;         // 0341  miss jump flag (witch space)
  unsigned char  cabin_temp;        // 0483  (C64, needed for trumble behaviour)
  
//; 0343  laser strength of fired laser
//; 0344  missile armed
//; 0346  "slow laser" - used to slow down rate of pulse and mining lasers
  unsigned char  screen;         // 87 is the current screen 
                                 // 0 for space view, bit masks for other screens

  signed char    fade_in;
  unsigned char    combat_seq;     // opening combat seq
  unsigned char    scan_zoom;      // scanner zoom - 0 == no zoom, 1 == zoom 

  unsigned char  shipset_key;    // current ship set loaded
  
  gametime_t time;

} player_stats_t;

extern commander_t g_commander;
extern commander_t tmp_save_commander; // this is set whenever we save the game
extern player_stats_t g_my_stats;
extern const commander_t default_commander;
extern const player_stats_t default_stats;
#endif

