#include <stdlib.h>
#include "screenmode.h"
#include "upgrade.h"
#include "motion.h"
#include "keypad.h"
#include "gbahandler.h"
#include "input.h"
#include "commander.h"
#include "flying.h"
#include "docked.h"
#include "eliteloop.h"
#include "navigation.h"
#include "options.h"
#include "reset.h"

#define B_ACTIVATE_MENU (3+g_game_options.frame_rate)
#define INFLIGHT_MENU_REPEAT  30
// don't change this or the docking computer may break!
#define CLIMB_DIVE_INCR 14

signed char g_virtual_keys[EVK_COUNT] = {};
signed char g_key_repeat[EVK_COUNT] = {};

static int g_max_turn = 0;
static int g_AB_off = 0;

void disable_AB(int count) {
  g_AB_off = count;
}

void dampen(int * value) {
  if (*value < 0) {
    (*value)++;
  }
  if (*value > 0) {
    (*value)--;
  }
}

void bump(int * value, int increment) {
  int v =  *value + increment;
  if (*value < 0) {
    v = 0;
  }
  if (v > 128) {
    v = 128;
  }
  *value = v;
}

void redu(int * value, int increment) {
  int v =  (*value) - increment;
  if ((*value) > 0) {
    v = 0;
  }
  if (v < -128) {
    v = -128;
  }
  *value = v;
}

int player_alpha = 0;
int player_beta = 0;
int player_delta = 0;

extern int ap_on;
void dampen_controls(int damp) {

  
  if (damp & 1) {
    dampen(&player_alpha);
    dampen(&player_alpha);
  } else {
    if (abs(player_alpha) > g_max_turn) {
      player_alpha = player_alpha<0?-(g_max_turn):g_max_turn;
    }
  }

  if (damp & 2) {
    dampen(&player_beta);
    // extra dampening, the BBC version seems too loose compared to NES/TNK
    dampen(&player_beta);
  } else {
    g_max_turn++;
    if (abs(player_beta) > g_max_turn) {
      player_beta = player_beta<0?-g_max_turn:g_max_turn;
    }
    g_max_turn--;
  }
  g_alpha = player_alpha/4;
  if (abs(g_alpha) < 8) {
    g_alpha /= 2;
  }

  g_beta = player_beta/16;
  
  if (abs(g_beta) < 3) {
    g_beta /= 2;
  }
  g_delta += player_delta;
  player_delta = 0;

  // check we are within ships range of movement
  if (g_delta > player_ship_data[g_commander.ship].max_speed) 
    g_delta = player_ship_data[g_commander.ship].max_speed;
  if (g_delta <= 0)
    g_delta = 1;

}

void zero_controls() 
{
  player_alpha = 0;
  g_alpha = 0;
  player_beta = 0;
  g_beta = 0;
  player_delta = 0;
  g_delta = 0;
}

void user_key_poll(int gba_key, int vk_code, int repeat) {
  if (KeyPressed(gba_key)) {
    g_key_repeat[vk_code]++;
    if (g_key_repeat[vk_code] > repeat)
      g_key_repeat[vk_code] = repeat;
    if (g_key_repeat[vk_code] > 1 && g_key_repeat[vk_code] < repeat) {
      g_virtual_keys[vk_code] = 0;
    } else {
      if (g_key_repeat[vk_code] != 1)
        g_key_repeat[vk_code] -= repeat/4;
      g_virtual_keys[vk_code] = 1;
    }
  } else {
    g_key_repeat[vk_code] = 0;
    g_virtual_keys[vk_code] = 0;
  }
}

void user_menu_controls(int repeat) {
  user_key_poll(KEY_R,EVK_MENU_R,repeat);
  user_key_poll(KEY_L,EVK_MENU_L,repeat);
  if (IS_PANEL_ON) {
    user_key_poll(KEY_A,EVK_MENU_A,repeat);
    user_key_poll(KEY_B,EVK_MENU_B,repeat);
  } else {
    g_virtual_keys[EVK_MENU_A] = 0;
    g_virtual_keys[EVK_MENU_B] = 0;
  }
  if (g_my_stats.docked)
    user_key_poll(KEY_START,EVK_MENU_START,repeat);

  if ( (!g_is_gba || KeyPressed(KEY_START)) && KeyPressed(KEY_SELECT) && KeyPressed(KEY_A) && KeyPressed(KEY_B))
  {
    reset_gba();
  }
}
void user_general_controls(int repeat) 
{
  user_key_poll(KEY_R,EVK_R,repeat);
  user_key_poll(KEY_L,EVK_L,repeat);
  user_key_poll(KEY_UP,EVK_UP,repeat);
  user_key_poll(KEY_DOWN,EVK_DOWN,repeat);
  user_key_poll(KEY_LEFT,EVK_LEFT,repeat);
  user_key_poll(KEY_RIGHT,EVK_RIGHT,repeat);
  if (g_AB_off == 0) {
    user_key_poll(KEY_A,EVK_A,repeat);
    user_key_poll(KEY_B,EVK_B,repeat);
  } else {
    g_virtual_keys[EVK_A] = 0;
    g_virtual_keys[EVK_B] = 0;
    g_AB_off--;
  }
}

void common_inflight_panel_controls(void) {
  user_menu_controls(INFLIGHT_MENU_REPEAT);
  if (g_virtual_keys[EVK_MENU_R]) {
    // increase selection
    inflight_panel_incrsel();
  } 
  if (g_virtual_keys[EVK_MENU_L]) {
    // increase selection
    inflight_panel_decrsel();
  } 
  if (g_virtual_keys[EVK_MENU_A]) {
    activate_inflight_selection();
  }
  if (g_virtual_keys[EVK_MENU_B]) {
    start_hide_docked_panel();
  }
}

void user_flight_controls(void) {
  // clear all key presses
  int i; 
  static int b_held = 0;
  for (i = 0; i < EVK_COUNT; i++)
    g_virtual_keys[i] = 0;
  
  if (g_hyperspace_counter == 0)  {
    // Roll
    if (KeyPressed(KEY_LEFT)) {
      g_virtual_keys[EVK_LEFT] = 1;
    }

    if (KeyPressed(KEY_RIGHT)) {
      g_virtual_keys[EVK_RIGHT] = 1;
    }

    if (KeyPressed(KEY_B)) {
      if (!IS_PANEL_ON) {
        // Speed
        if (KeyPressed(KEY_UP)) {
          g_virtual_keys[EVK_ACCEL] = 1;
        }
        if (KeyPressed(KEY_DOWN)) {
          g_virtual_keys[EVK_ACCEL] = -1;
        }
        b_held++;
        if (b_held > 0x100)
          b_held = 0x100;
      } else {
        // panel off
      }
    } else {
      // Pitch
      if (KeyPressed(KEY_UP)) {
        g_virtual_keys[EVK_UP] = 1;
      }
      if (KeyPressed(KEY_DOWN)) {
        g_virtual_keys[EVK_DOWN] = 1;
      }
      if (b_held) {
        if (b_held <= B_ACTIVATE_MENU && g_AB_off == 0) {
          int panel_list[15];
          get_active_panels(panel_list);
          activate_inflight_selection();
        }
        b_held = 0;
      }
    }

    if (!IS_PANEL_ON) {
      if (KeyPressed(KEY_A) && g_AB_off == 0) {
        // fire our laser
        g_virtual_keys[EVK_A] = 1;
      }
    }
    
#ifdef ALLOW_RENDER_CHANGE
    // test destroying items
    if (KeyPressedNoBounce(KEY_SELECT,3) ) {
      extern int g_RENDER_AS_TRIANGLES;
      g_RENDER_AS_TRIANGLES ^= 1;
    }
#endif
    if (!IS_PANEL_ON) {
      if (KeyPressedNoBounce(KEY_SELECT,3)) {
#if 0
        static int message_id = 0;
        static const int const_messages[] = {
          0x6e + 6,
          0x64, 0x7B, 0xA0, 0xC9, IFF_FLIGHT_MESSAGE( 52 ), 
          0x78, 0xc8, 0x65, 0xd0, 0x0, 
          IFF_FLIGHT_MESSAGE(7),
          IFF_FLIGHT_MESSAGE(9),
          IFF_FLIGHT_MESSAGE(17),
          IFF_FLIGHT_MESSAGE(27),
          IFF_FLIGHT_MESSAGE(47),
          
        };
        extern int g_message_after;
        if (message_id == 0)
          g_message_after = 1;
        flight_message(const_messages[message_id++]);
        
        if (message_id == 16)
          message_id = 0;
#endif
        if (g_my_stats.screen == SCR_LAUNCHED) {
          g_galactic_hyper_preliminary = 180;
        }
      }
    } 

    common_inflight_panel_controls();
    // if start pressed, pause game
    // if select pressed... erm...
  }
  if (g_AB_off)
    --g_AB_off;

}

void update_motion(void) {
  g_max_turn = 0x7f - (player_ship_data[g_commander.ship].turning_circle^0xff);
  int damp = 3;
  if (g_virtual_keys[EVK_LEFT]) {
    bump( &player_alpha, 7);
    if (g_virtual_keys[EVK_LEFT]>1) {
      bump( &player_alpha, 7);
      if (g_virtual_keys[EVK_LEFT]>2) {
        bump( &player_alpha, 7);
      }
    }
    damp &= ~1;
  } 
  if (g_virtual_keys[EVK_RIGHT]) {
    redu( &player_alpha, 7);
    if (g_virtual_keys[EVK_RIGHT]>1) {
      redu( &player_alpha, 7);
      if (g_virtual_keys[EVK_RIGHT]>2) {
        redu( &player_alpha, 7);
      }
    }
    damp &= ~1;
  }
  if ( g_virtual_keys[EVK_ACCEL]) {
    player_delta = g_virtual_keys[EVK_ACCEL];
  }
  if ( g_virtual_keys[EVK_UP]) {
    redu( &player_beta, CLIMB_DIVE_INCR);
    damp &= ~2;
  }
  if ( g_virtual_keys[EVK_DOWN]) {
    bump( &player_beta, CLIMB_DIVE_INCR);
    damp &= ~2;
  }

  dampen_controls(damp);

  // hack to get the final stage rotation working
  if (g_virtual_keys[EVK_RIGHT] == 0x7f) {
    g_alpha = -15;
    player_alpha = -17*4;
  }  

  if (g_virtual_keys[EVK_A] && ( g_my_stats.screen == SCR_LAUNCHED) )
    fire_laser();

}

// so that pressing A doesn't skip screens
void wait_letgo_A() 
{
  while (KeyPressed(KEY_A)) {
    UpdateKeys();
    wait_vsync_int();
  }
}


