
#include "screenmode.h"
#include "gfx.h"
#include "docked.h"
#include "input.h"
#include "commander.h"
#include "eliteloop.h"
#include "clipping.h"
#include "planet.h"
#include "universe.h"
#include "stars.h"
#include "dialgauge.h"
#include "flying.h"
#include "soup.h"
#include "text.h"
#include "bay.h"
#include "gbahandler.h"
#include "colours.h"
#include "soundfx.h"
#include "keypad.h"
#include "status.h"
#include "trumbles.h"

#include "sxmp.h"

#define FRONT_VIEW 0
#define FRONT_STRING 0x60
#define VIEW_STRING 0xaf

void look1(void) {
  // 5478 
  // if a new look, like after launch
  //  clear screen
  //  draw sight
  show_crosshair(g_commander.equipment[g_my_stats.view]);
  view_trumbles();
  // draw missiles
  set_missile_count(g_commander.equipment[MISSILES]);
  if (g_my_stats.missile_armed == -1) {
    arm_missile(); 
  } else if (g_my_stats.missile_armed != 0) {
    lock_missile();
  }

  if (g_object_types[SHIP_SST]) {
    activate_sprites(SST_SPRITE, SST_SPRITE+1);
  } 
  if (g_my_stats.ecm_counter) {
    activate_sprites(ECM_SPRITE, ECM_SPRITE+1);
  }
  if (g_my_stats.miss_jump) {
    deactivate_sprites(COMPASS_SPRITE, COMPASS_SPRITE+1);
  }
  //  new stars
  g_my_stats.screen = 0;
  update_stars(0);
  dial(DIAL_FU, g_commander.fuel/2);
  update_dials();
  
  if (g_my_stats.miss_jump==0) {
    update_altitude(0);
    update_altitude(1);
  }
  // draw view text
  set_text_x(11);
  g_yc = 1;
  textmod = 0x80;
  text2buffer(g_my_stats.view|FRONT_STRING);
  text2buffer(' ');
  text2buffer(VIEW_STRING);
  
}

// this should be elsewhere...
#define MAX_CIRCLES 6
void launch_pattern (int start_speed, int step)
{
  // uses step to speed up on launch, slow down when dock
  signed short circle_z[MAX_CIRCLES];

  unsigned char i,j;
  const int size = 40;

  for (i = 0; i < MAX_CIRCLES; i++) {
    circle_z[i] = -i*size;
  }

  clear_screen();
  i = 0;
  int increment = start_speed*32;
  while (i == 0) {
    //lineColour = 1;
    for (j = 0; j < MAX_CIRCLES; j++) {
      if (circle_z[j] < SCREEN_SIZE_X) {
        //radius = 1000/circle_z[j];
        if ( circle_z[j] > 0) {
          lineColour = VECTOR_COL;
          g_stp = 8;
          circle2(SCREEN_SIZE_X, SCREEN_SIZE_Y, circle_z[j]);
        }
        circle_z[j] += (increment/32);
        //if (j == (MAX_CIRCLES -1) && circle_z[j] > (SCREEN_SIZE_X-8)) {
        //  update_universe();
        //update_stars();
        //}
      } else if (j == (MAX_CIRCLES - 1) ) {
        i = 1;
      } 
    }
    wait_vsync_int();
    swap_buffer();
    clear_screen();
    increment += step;

  }
}

void launch(void) {
  // this is also "front view"
  
  g_my_stats.auto_pilot = 0;
  // show launch pattern
  g_my_stats.screen = SCR_LAUNCHED;
  init_bgs();
  // clear universe, set roll and pitch to 0
  zero_objects();
  // set hyperspace planet from coords
  hyp1(0);

  // set the random seed here based on game_frames 
  g_rand_seed.r1 = game_frames&0xff;
  g_rand_seed.r3 = (game_frames>>8)&0xff;

  // this is just an educated guess:
  // set random ship encounters
  load_shipset();
  
  // add planet object at 0x010000
  add_planet();
  // draw missile blobs
  g_key_handler = user_flight_controls;
  g_timer_fn = space_screen;
  // add space stat at -0x0100
  add_station();
  // set speed to 12
  g_delta = 12;
  player_alpha = player_beta = 0;
  g_alpha = 12;
  player_alpha= 12*4;
  // "increase" badness
  g_commander.legal_status |= illegal_cargo_value();
  // look1 view = 0
  g_my_stats.view = FRONT_VIEW;
  // show altitude gauge
  dial(DIAL_AL, 29);
  play_sound(SOUNDFX_LAUNCH);
  look1();
  launch_pattern(2, 1);
  UpdateKeys();
  user_flight_controls();
  g_my_stats.docked = 0;
}

void set_view(int view) {
  g_new_stars = 0;
  g_my_stats.view = view;
  g_key_handler = user_flight_controls;
  g_timer_fn = space_screen;
  g_panel_timer = 0;
  g_my_stats.screen = SCR_LAUNCHED;
  screen_fade(16);
  init_bgs();
  swap_buffer();
  look1();
  update_universe();
  g_my_stats.fade_in = 16;
}

void set_front_view(void) {
  set_view(FRONT_VIEW);
}
void set_rear_view(void) {
  set_view(REAR_VIEW);
}
void set_left_view(void) {
  set_view(LEFT_VIEW);
}
void set_right_view(void) {
  set_view(RIGHT_VIEW);
}

// we have docked
void dock_player(void) {
  sxmp_fade_out();
  hide_docked_panel();     
  reset_lollipops();
  zero_objects();
  zero_controls();
  if (g_my_stats.auto_pilot & 2) {
    // we have used the rent-a-dock feature
    g_commander.cash -= RENT_A_DOCK_PRICE;
  }
  // the docking bay sequence
  play_sound(SOUNDFX_LAUNCH);
  launch_pattern(5, -1);
  // show the parked ships...
  show_bay();
  reset_stats();
  bay();
  // auto tmp save
  tmp_save_commander = g_commander;
}


