#include <string.h>
#include "screenmode.h"
#include "keypad.h"
#include "sprite.h"
#include "interrupts.h"
#include "saving.h"
#include "text.h"
#include "commander.h"
#include "menu.h"
#include "gfx.h"
#include "keyboard.h"
#include "loadsave.h"
#include "soup.h"
#include "options.h"
#include "intro2.h"
#include "gfx.h"
#include "input.h"
#include "gbahandler.h"

#define SAVE_GAME_STR  22
#define LOAD_GAME_STR  23
#define DELETE_GAME_STR  54
#define SAVE_COUNT 8
#define SAVE_MENU_Y 4
#define SAVE_MENU_X 8
#define QUICK_SAVE  9
#define QUICK_SAVE_STRING  55

//void bytecpy(void * dest, const void *src , unsigned int size) LONG_CALL;

const char * MAGIC_COOKIE = "ELA";

int g_on_savescrn = 0;

// save structure mapped in SRAM
typedef struct {
  char used;
  char pad;
  unsigned char checksum_eor;
  unsigned char checksum;
  gametime_t time;
  commander_t commander;
} commander_save_t;

typedef struct
{
  commander_save_t commander_save;
  player_stats_t   stats;
  stellarobject_t  objects[NUM_SHIPS];
  int    types[MAX_SHIP_ID];
  int delta;
  int alpha;
  int beta;
} quicksave_t;

typedef struct {

  int pad; // don't use first bytes
  char magic_cookie[4];
  
  commander_save_t saves[SAVE_COUNT];
  game_options_t options;

  quicksave_t quicksave;
}
save_t;

save_t * sram_saves = (save_t*) (SRAM);

int checksum(const unsigned char * block, int save_size) {
  // actual saving needs to use checksum
  // 1245]CH%=NT%-2
  //      CY%=0
  //      FOR I% = CH%+J% TO 1+J% STEP-1
  //          CH%=CH%+CY%+(I%?7)
  //          CY%=(CH%>255)AND1
  //          CH%=CH%MOD256
  //          CH%=CH%EOR(I%?8) <-- +8 to skip name?
  //      NEXT
  //     [OPTZ
  save_size--;
  int ch = save_size;
  int cy = 0;
  int i;
  for (i = save_size ; i != 1; i--) {
    ch = ch + cy + block[i-1];
    cy = (ch>255)&1;
    ch = ch%256;
    ch ^= block[i];
  }
  return ch;
}

// this should be in ewram according to the docs?
void bytecpy(unsigned char * dest, const unsigned char *src, unsigned int size) {
  while (size) {
    *dest++ = *src++;
    size--;
  }
}

static void save_number(int i) {
  static const char *val = "12345678";
  char str[2] = {val[i],0}; 
  vw_print_str(str, SAVE_MENU_X, i+SAVE_MENU_Y);
  set_text_x(SAVE_MENU_X+3);
}

// show an empty slot:
void empty_save(int i) {
  // print i. -------- at i+SAVE_MENU_Y
  if (i < QUICK_SAVE) {
    save_number(i);
    vw_print_str("--------", SAVE_MENU_X+2, i+SAVE_MENU_Y);
  } 
}

// changed ot pointer / id for quick save
void show_save(commander_save_t * save, int i) {
  if (save->used) {
    // display save name
    save_number(i);
    char buffer[8];
    bytecpy(buffer, (unsigned char*)save->commander.name, 8);
    vw_print_str(buffer, SAVE_MENU_X+2, i+SAVE_MENU_Y);
    g_yc = SAVE_MENU_Y+i;
    set_text_x(20);
    gametime_t time;
    bytecpy((char*)&time, (unsigned char*)&save->time, sizeof(gametime_t));

    print_number(time.hour, 4);
    handle_letter(':');
    print_number_pad(time.minute, 2, '0');
    handle_letter(':');
    print_number_pad(time.second, 2, '0');
  } else {
    empty_save(i);
  }
}

int get_have_saves() 
{
  char mc[4];
  bytecpy(mc, (unsigned char*)sram_saves->magic_cookie, 3);
  mc[3] = 0;
#ifndef USE_STRCMP
  // to compare a 4 byte string, can place it in an int
  u32 * mci = (u32*)mc;
  const u32 * cMCi = (const u32*)MAGIC_COOKIE;
  if ((*mci) == (*cMCi))
#else
  if (0 == strcmp(mc, MAGIC_COOKIE)) 
#endif
  {
    return 1;
  }
  return 0;
}

void create_save_screen();
void create_list() {
  SetInterrupt(0);
  clear_text();

  g_yc = SAVE_MENU_Y-2;
  textmod = NAME_CASE;
  set_text_x(SAVE_MENU_X+2);
  text2buffer(0xfc);
  set_text_x(22);
  custom2buffer(18);
  set_row_pal(g_yc, TEXT_PAL_CYAN);
  
  // first check cookie exists
  if (!get_have_saves()) {
    save_options();
  }
  // list saves
  int i;
  for (i = 0 ; i < SAVE_COUNT; i++) {
    show_save(&sram_saves->saves[i], i);
  }
  SetInterrupt(1);
}

void copy_save_to_sram(commander_save_t * save)
{
    SetInterrupt(0);
    // save ch^0xa9 and ch
    // save g_commander in chosen slot
    bytecpy((unsigned char*)&save->commander, (unsigned char*)&g_commander, sizeof(commander_t));
    int ch = checksum((unsigned char*)&g_commander, sizeof(commander_t));

    save->checksum = (char)ch;
    save->checksum_eor = (char)(ch^0xa9);
    save->used = 1;
    bytecpy((unsigned char*)&save->time,
        (unsigned char*)&g_my_stats.time, 
        sizeof(gametime_t)
        );
    SetInterrupt(1);
}

void perform_save() {
  int index = get_menu_selection();
  // redraw list
  deactivate_sprites(0, 128);
  CopyOAM();
  if (sram_saves->saves[index].used == 0 || yes_no_selection(50,3,15)) {

    // only copy 8 bytes of the name
    bytecpy(g_commander.name, g_keyboard_str, 8);

#if 0
    SetInterrupt(0);

    // save ch^0xa9 and ch
    // save g_commander in chosen slot
    bytecpy((unsigned char*)&sram_saves->saves[index].commander, (unsigned char*)&g_commander, sizeof(commander_t));
    int ch = checksum((unsigned char*)&g_commander, sizeof(commander_t));

    sram_saves->saves[index].checksum = (char)ch;
    sram_saves->saves[index].checksum_eor = (char)(ch^0xa9);
    sram_saves->saves[index].used = 1;
    bytecpy((unsigned char*)&sram_saves->saves[index].time,
        (unsigned char*)&g_my_stats.time, 
        sizeof(gametime_t)
        );
    tmp_save_commander = g_commander;
    g_need_combat_seq = 0; 
    SetInterrupt(1);
#endif
    copy_save_to_sram(&sram_saves->saves[index]);
    tmp_save_commander = g_commander;
    g_need_combat_seq = 0; 
  }
  create_save_screen();
  set_menu_selection(index);
  // reset the panel menu as it has been overwritten by the vkeys
  load_docked_panel();
}
void update_savename(void) {
  clear_row(19);
  g_xc = 1; g_yc = 19;
  soup2buffer(0xc8); // Name?
  set_text_x(11);
  vw_print_str(g_keyboard_str, g_xc, 19);
  
  if (g_keyboard_str_index < 8 && ((game_frames&15) < 10) )
    vw_print_str("_", g_xc, 19);
}


int save_a(int index) {
  //g_on_savescrn = 0;
  // virtual keyboard to change the name
  load_vk('>');
  // on "action" should save, otherwise just show the options screen
  set_vk_callbacks(update_savename,perform_save, SCR_OPTIONS);
  g_key_handler = virtual_keys;
  bytecpy(g_keyboard_str, g_commander.name, 8);
  g_keyboard_str_index = 0;
  while (g_commander.name[g_keyboard_str_index]) 
    g_keyboard_str_index++;
  update_savename();

  return index;
}

void restore_commander(const commander_t * tmp) 
{
  g_commander = *tmp;
  // restore planet coords and market prices
  hyp1(1);
  calculate_market();
  g_commander = *tmp;
  tmp_save_commander = g_commander;
}

void restore_save(commander_save_t * save)
{
  if (save->used) {
    commander_t tmp;
    bytecpy((unsigned char*)&tmp, (unsigned char*)&save->commander, sizeof(commander_t));
    // checksum
    int ch = checksum((unsigned char*)&tmp, sizeof(commander_t));
    int ch_eor = ch^0xa9;
    if (ch == save->checksum &&
        ch_eor == save->checksum_eor ) 
    {
      // ok
      g_on_savescrn = 0;
      g_need_combat_seq = 0; 
      restore_commander(&tmp);
      bytecpy((unsigned char*)&g_my_stats.time, 
          (unsigned char*)&save->time, 
          sizeof(gametime_t)
          );
    } else {
      g_on_savescrn = 1;
      menu_cerror("CHECKSUM?");
    }
  }
}

void restore_quicksave()
{
  if (sram_saves->quicksave.commander_save.used) {
    // it is used, try restoring it
    restore_save(&sram_saves->quicksave.commander_save);
    sram_saves->quicksave.commander_save.used = 0;
    if (g_on_savescrn==0) {
      // loaded OK. now we need to load the player stats, objects and types
      bytecpy((unsigned char*)&g_my_stats, (unsigned char*)&sram_saves->quicksave.stats, sizeof(player_stats_t));
      bytecpy((unsigned char*)g_objects, (unsigned char*)sram_saves->quicksave.objects, sizeof(g_objects));
      bytecpy((unsigned char*)g_object_types, (unsigned char*)sram_saves->quicksave.types, sizeof(g_object_types));

      bytecpy((unsigned char*)&g_delta,   (unsigned char*)&sram_saves->quicksave.delta, sizeof(int));
      bytecpy((unsigned char*)&g_alpha,   (unsigned char*)&sram_saves->quicksave.alpha, sizeof(int));
      bytecpy((unsigned char*)&g_beta,    (unsigned char*)&sram_saves->quicksave.beta, sizeof(int)); 

      int ssk = g_my_stats.shipset_key;
      do_load_shipset(&ssk);
#if 0
      int i;
      for (i = 0; i < NUM_SHIPS; i++) {
        gbaprintf("g_objects[%d].type = %d\n", i, g_objects[i].type);
        gbaprintf("g_objects[%d].location = %d %d %d\n", i, g_objects[i].location.x,  g_objects[i].location.y,  g_objects[i].location.z);
      }
      gbaprintf("g_my_stats.game_over %d\n", g_my_stats.game_over);
#endif
    }
  }
}

int load_a(int index) {
  SetInterrupt(0);
  if (index == SAVE_COUNT) {
    g_on_savescrn = 0;
    restore_commander(&default_commander);
    g_need_combat_seq = 1; 
    gametime_t zero = {0,0,0,0};
    g_my_stats.time = zero;
  } else if (index == QUICK_SAVE) {
    // load quicksave
    restore_quicksave();
  } else {
    restore_save(&sram_saves->saves[index]);
  }
  SetInterrupt(1);
  return index;
}
int load_b(int index) {
  g_on_savescrn = 0;
  return 0;
}
int load_moreinfo(int index) {
  return index;
}

void general_sram_screen(int title, MenuFunctionPtr func_a) 
{
  g_my_stats.screen = SCR_OPTIONS;
  set_bay_bgs();
  create_list();
  draw_border();
  g_yc = 0;
  g_xc = 11;

  textmod = ALL_CAPS;
  custom2buffer(title);
  set_row_pal(g_yc,TEXT_PAL_YELLOW); 
  set_menu_callbacks(
      load_moreinfo,
      load_moreinfo,
      func_a,
      load_b
      );
}

void restore_game() {
  // a screen to restore saved game
  general_sram_screen(LOAD_GAME_STR, load_a);

  // default jameson
  set_text_x(SAVE_MENU_X);
  g_yc = SAVE_COUNT+SAVE_MENU_Y;
  custom2buffer(17);
  vw_print_str(default_commander.name, g_xc, SAVE_COUNT+SAVE_MENU_Y);
  reset_menu(SAVE_COUNT+2, SAVE_MENU_Y);

  g_yc++;
  if (!sram_saves->quicksave.commander_save.used) {
    set_text_x(SAVE_MENU_X);
    custom2buffer(49);
    
    set_text_x(g_xc+1);
    textmod = NAME_CASE;
    custom2buffer(QUICK_SAVE_STRING);
    set_row_pal(g_yc, TEXT_PAL_GREY);
  } else {
    set_text_x(1);
    custom2buffer(QUICK_SAVE_STRING);
    show_save(&sram_saves->quicksave.commander_save, QUICK_SAVE);
  }
  g_on_savescrn = 1;

  enable_menu(1);
  while (g_on_savescrn && g_my_stats.screen == SCR_OPTIONS) {
    wait_vsync_int();
    UpdateKeys();
    menu_handler();
    menu_msg_timer();
  }
  g_on_savescrn = 0;
}

void create_save_screen()
{
  // a screen to save the game..
  general_sram_screen(SAVE_GAME_STR, save_a);
  reset_menu(SAVE_COUNT, SAVE_MENU_Y);
}

void save_game() {
  create_save_screen();
  g_on_savescrn = 1;
  enable_menu(1);
  while (g_on_savescrn && g_my_stats.screen == SCR_OPTIONS) {
    wait_vsync_int();
    UpdateKeys();
    g_key_handler();
    menu_msg_timer();
  }
  g_on_savescrn = 0;
  
}

int delete_a(int index) 
{
  if (sram_saves->saves[index].used) {
    if (yes_no_selection(53,3,16)) {
      sram_saves->saves[index].used = 0;
    } 
    g_on_savescrn = 0;
  } 
  return 0;
}
void delete_game() {
  // a screen to delete saved game
  general_sram_screen(DELETE_GAME_STR, delete_a);

  reset_menu(SAVE_COUNT, SAVE_MENU_Y);

  g_on_savescrn = 1;
  enable_menu(1);
  while (g_on_savescrn && g_my_stats.screen == SCR_OPTIONS) {
    wait_vsync_int();
    UpdateKeys();
    menu_handler();
    menu_msg_timer();
  }
}

void save_options() 
{
  SetInterrupt(0);
  if (!get_have_saves()) {
    bytecpy(sram_saves->magic_cookie , MAGIC_COOKIE,3);
    // also make sure all saves are not used
    int i;
    for (i = 0 ; i < SAVE_COUNT; i++) {
      sram_saves->saves[i].used = 0;
    }
  }
  bytecpy((unsigned char*)&sram_saves->options, (unsigned char*)&g_game_options, sizeof(game_options_t));
  SetInterrupt(1);
}

void load_options() 
{
  SetInterrupt(0);
  if (get_have_saves()) {
    bytecpy((unsigned char*)&g_game_options, (unsigned char*)&sram_saves->options, sizeof(game_options_t));
  } else {
    g_my_stats = default_stats;
    save_options();
  }
  SetInterrupt(1);
}

void quick_save()
{
  // perform quick save...
  g_my_stats.screen = SCR_OPTIONS;
  set_bay_bgs();
  draw_border();
  g_yc = 0;
  g_xc = 11;

  textmod = ALL_CAPS;
  custom2buffer(QUICK_SAVE_STRING);
  set_row_pal(g_yc,TEXT_PAL_YELLOW); 
  
  set_text_x(3);
  g_yc = 9;
  custom2buffer(QUICK_SAVE_STRING);
  custom2buffer(QUICK_SAVE_STRING+1);
  
  if (yes_no_selection(50,3,11)) {
    // quick save the game...
    copy_save_to_sram(&sram_saves->quicksave.commander_save);
    SetInterrupt(0);
    bytecpy((unsigned char*)&sram_saves->quicksave.stats,  (unsigned char*)&g_my_stats, sizeof(player_stats_t));
    bytecpy((unsigned char*)sram_saves->quicksave.objects, (unsigned char*)g_objects, sizeof(g_objects));
    bytecpy((unsigned char*)sram_saves->quicksave.types,   (unsigned char*)g_object_types, sizeof(g_object_types));
    bytecpy((unsigned char*)&sram_saves->quicksave.delta,   (unsigned char*)&g_delta, sizeof(int));
    bytecpy((unsigned char*)&sram_saves->quicksave.alpha,   (unsigned char*)&g_alpha, sizeof(int));
    bytecpy((unsigned char*)&sram_saves->quicksave.beta,   (unsigned char*)&g_beta, sizeof(int));
    SetInterrupt(1);
    g_my_stats = default_stats;
    g_my_stats.game_over = 2;
    g_my_stats.combat_seq = 0;
    screen_fade_out();
#if 0
    int i;
    for (i = 0; i < NUM_SHIPS; i++) {
      gbaprintf("g_objects[%d].type = %d\n", i, g_objects[i].type);
      gbaprintf("g_objects[%d].location = %d %d %d\n", i, g_objects[i].location.x,  g_objects[i].location.y,  g_objects[i].location.z);
    }
#endif
  } 
  g_on_savescrn = 0;
}
