#include <stdio.h>
#include <stdlib.h>
#include "screenmode.h"
#include "keypad.h"
#include "gbahandler.h"
#include "text.h"
#include "navigation.h"
#include "galaxy.h"
#include "soup.h"
#include "gfx.h"
#include "commander.h"
#include "docked.h"
#include "universe.h"
#include "eliteloop.h"
#include "input.h"
#include "stars.h"
#include "tactics.h"
#include "launch.h"
#include "flying.h"
#include "soundfx.h"
#include "scanner.h"
#include "missions.h"
#include "dialgauge.h"
#include "trumbles.h"


#define HYPERSPACE_COUNTER_X    1
#define HYPERSPACE_COUNTER_Y    1
#define HYPERSPACE_INFO_X    6
#define HYPERSPACE_INFO_Y    12
#define HYPERSPACE_COUNTDOWN     15

#define DATA_ON_BLOCK_1_X  10
#define DATA_ON_BLOCK_2_X  16

#define HYPERSPACE_STRING 0xBD
#define CH_PLANET_STRING  0x3

#define MISS_JUMP_CHANCE  0xfd
#define DRIVE_MALFUNCTION  52

// counter is the number shown
int g_hyperspace_counter = 0;
// timer is the frames between number changes
int g_hyperspace_timer = 0;

planetinfo_t hyp_info = {};
planetinfo_t cpl_info = {};

int g_last_found = 0;
void get_planet_at(int x, int y, seed_t * found_seed) {

  int distance = 0xff;
  int count = 0;
  // work with a temp copy
  seed_t tmp_seed = g_commander.gal_seed;
  //int y_found = 0;
  
  for (count = 0; count < 256; count++) {
    
    int dx = abs(tmp_seed.s3 - x);
    int dy = abs(tmp_seed.s1 - y);

    //int dxdy = dx/2 + dy/2; // buggy - find planet "laeden" to see
    int dxdy = dx + dy;
    
    if (dxdy < distance) {
      g_last_found = count;
      distance = dxdy;
      *found_seed = tmp_seed;
    }
    churn_seed(&tmp_seed);
    churn_seed(&tmp_seed);
    churn_seed(&tmp_seed);
    churn_seed(&tmp_seed);
  }
}

void set_hyp_info() {
  // set hyperspace planet info
  int dx = abs(hyp_coord.x - g_commander.cpl_coord.x);
  int dy = abs(hyp_coord.y - g_commander.cpl_coord.y)/2;
  int Q = bios_sqrt(dx*dx + dy*dy);

  hyp_info.distance = Q*4;
  hyp_info.economy  = hyp_seed.s1&7;
  hyp_info.gov      = (hyp_seed.s2/8)&7;
  if ( (hyp_info.gov/2) == 0) {
    hyp_info.economy |= 2;
  }
  hyp_info.tech     = (hyp_info.economy^7) + (hyp_seed.s3&3);
  hyp_info.tech     += (hyp_info.gov/2) + (hyp_info.gov&1); // carry in ADC after LSR

  int A = hyp_info.tech * 4;
//  int carry = (A&0x100)!=0;
//  A &= 0xff;
  A += hyp_info.economy; // + carry;

//  carry = (A&0x100)!=0;
//  A &= 0xff;
  A += hyp_info.gov; // + carry;
    
//  carry = (A&0x100)!=0;
//  A &= 0xff;
  A += 1; //+ carry;
//  A &= 0xff;

  hyp_info.population  = A;
  hyp_info.gross  = 8 * hyp_info.population * 
    (3 + (hyp_info.economy^7)) * (4 + hyp_info.gov);

}

void hyp1(int first) {
  if (first) {
    get_planet_at(g_commander.cpl_coord.x, g_commander.cpl_coord.y, &hyp_seed);
    cpl_seed = hyp_seed;
    g_commander.cpl_coord.x = hyp_seed.s3;
    g_commander.cpl_coord.y = hyp_seed.s1;
  } else {
    get_planet_at(hyp_coord.x, hyp_coord.y, &hyp_seed);
  }
  hyp_coord.x = hyp_seed.s3;
  hyp_coord.y = hyp_seed.s1;
  set_hyp_info();
  if (first) {
    // hyperspace here, so copy its info
    cpl_info = hyp_info;
  }
}



void christen_planet(seed_t tmpseed) {
  // .cpl or 2DE6 
  // copy seed to tmp holder, then work with this
//  seed_t tmpseed = *seed;
  int cnt = 3;
  cnt-= ((tmpseed.s0 & 0x40) == 0);
  do {
    int A = tmpseed.s5&0x1f;
    if (A) {
      text2buffer(A|0x80);
    }
    churn_seed(&tmpseed);
  } while ( cnt-- );
}


#define GOVERNMENT     0xa2
#define DATA_ON_CPL    0xa3
#define ECONOMY_TYPE   0xa8
#define ECONOMY_CLASS  0xaa
#define MAINLY         0xad
#define GOV_TYPE       0xb1
#define HUMAN_COLONIAL 0xbc
#define DISTANCE       0xbf
#define POPULATION     0xc0
#define GROSS_PROD     0xc1
#define ECONOMY        0xc2
#define TECH_LEVEL     0xc4
#define BILLION        0xc6
#define TEXT_CR        0xe2
#define FIERCE         0xe3
#define CREATURE_TYPE  0xe6
#define BUG_EYED       0xec
#define RODENT         0xf2
#define AVERAGE_RADIUS 0xfa

void show_human_colonial(void) {
  //g_bindex = 0;
  g_xc = 2;
  text2buffer('(');
  textmod = NAME_CASE;
  if (hyp_seed.s4 & 0x80) {
    // bug-eyed rabbits
    int ct = (hyp_seed.s5/4)&7;
    if (ct < 3) {
      text2buffer(FIERCE+ct);
      text2buffer(' ');
      textmod = NAME_CASE;
    }
    ct = (hyp_seed.s5/32);
    if (ct < 6) {
      text2buffer(CREATURE_TYPE+ct);
      text2buffer(' ');
      textmod = NAME_CASE;
    }
    ct = (hyp_seed.s3^hyp_seed.s1)&7;
    if (ct < 6) {
      text2buffer(BUG_EYED+ct);
      text2buffer(' ');
      textmod = NAME_CASE;
    }
    ct += (hyp_seed.s5&3);
    ct &= 7;
    text2buffer(RODENT+ct);


  } else {
    text2buffer(HUMAN_COLONIAL);
  }
  text2buffer('S');
  print_lf(')');
  
}


void show_distance(void) {
  if (hyp_info.distance > 0) {
    textmod = NAME_CASE;
    //print_dots(DISTANCE);
    text2buffer(DISTANCE);
    //printf("%s %d.%d",g_buffer,hyp_info.distance/10,hyp_info.distance%10);
    //print_buffer(g_xc, g_yc);
    //set_text_x(DATA_ON_BLOCK_1_X);
    print_decimal(hyp_info.distance,4);
    textmod = NAME_CASE;
    print_lf(LIGHT_YEARS);
  }
}

int g_planet_back = 0;
void planet_keys(void) {
  int on = IS_PANEL_ON;
  if (g_my_stats.docked)
    common_docked_keys();
  else
    common_inflight_panel_controls();
  // rest of keys to move cursor around, etc
  if (!on) {
    user_general_controls(90);
    //if (KeyPressedNoBounce(KEY_B, 4)) 
    if (g_virtual_keys[EVK_B])
    {
      if (g_planet_back) {
        manual_activate_screen(g_planet_back);
        g_planet_back = 0;
        disable_AB(30);
      }
    }
  }
}

void show_current_planet(void) {
  g_my_stats.screen = SCR_PLANET_INFO;
  // 23EA in title.asm
  // draw a frame
  //g_bindex = 0;
  g_xc = 9;
  g_yc = 0;
  textmod = ALL_CAPS;
  g_text_palette = TEXT_PAL_YELLOW;
  print_lf(DATA_ON_CPL);
  g_text_palette = TEXT_PAL_WHITE;
  g_yc++;
  show_distance();
  
  //g_bindex = 0;
  textmod = NAME_CASE;
  //print_dots(ECONOMY);
  text2buffer(ECONOMY);
  //set_text_x(DATA_ON_BLOCK_1_X);
  int ectype = (hyp_info.economy+1);
  int carry = ectype&1;
  ectype/=2;
  if (ectype == 2) {
    text2buffer(MAINLY);
    textmod = NAME_CASE;
  } else {
    if (ectype > 2) {
      ectype = hyp_info.economy - 5; // - !carry;
      carry = 0;
    } 
    ectype += ECONOMY_CLASS + carry;  // rich/average/poor
    text2buffer(ectype);
    textmod = NAME_CASE;
  }
  print_lf_name(ECONOMY_TYPE + (hyp_info.economy/4) );
  
  text2buffer(GOVERNMENT);
  text2buffer(0x9);
  text2buffer(0x8);
  //set_text_x(DATA_ON_BLOCK_1_X);
  print_lf_name(GOV_TYPE + hyp_info.gov);

  text2buffer(TECH_LEVEL);
  //set_text_x(DATA_ON_BLOCK_1_X);
  print_number(hyp_info.tech+1,2);
  print_lf(LF);

  //textmod = NAME_CASE;
  text2buffer(POPULATION);
  //set_text_x(DATA_ON_BLOCK_1_X);
  print_decimal(hyp_info.population,3);
  textmod = NAME_CASE;
  print_lf_name(BILLION);
  
  show_human_colonial();
  text2buffer(LF);
  
  textmod = NAME_CASE;
  text2buffer(GROSS_PROD);
  //set_text_x(DATA_ON_BLOCK_2_X);
  print_number(hyp_info.gross,4);
  textmod = NAME_CASE;
  text2buffer(' ');
  text2buffer('M');
  textmod = NAME_CASE;
  print_lf_name(TEXT_CR);

  text2buffer(AVERAGE_RADIUS);
  set_text_x(DATA_ON_BLOCK_2_X);
  int radius = ((0xb + (hyp_seed.s5 & 0xf))<<8) | hyp_seed.s3;
  print_number(radius,4);
  textmod = LOWER_CASE;
  text2buffer(' ');
  text2buffer('K');
  text2buffer('M');
  print_lf(LF);

  if (hyp_info.distance)
    style_sheet(3);
  else
    style_sheet(1); 
  // Lave is famous for...
  // fill in mission text - if no mission, print goat soup
  if (!have_mission_description())
    goat_soup(&hyp_seed);
  
  g_key_handler = planet_keys;
}

void do_hyperspace(void) {
  // 3201
  // make sure we set our coords to the hyp planet
  hyp1(0);
  // now copy our hyperspace stuff to the current planet
  cpl_seed = hyp_seed;
  g_commander.cpl_coord = hyp_coord;
  // the infos still contain the cpl and hyp values...

  reset_scanner_options();
  
  cpl_info.economy = hyp_info.economy;
  cpl_info.gov = hyp_info.gov;
  cpl_info.tech = hyp_info.tech;

  g_commander.planet_random = get_rand();
  calculate_market();
  // clear out the universe
  zero_objects();
  // in classic Elite this was status /= 2
  int nls = g_commander.legal_status - (hyp_info.distance/8);
  if (nls < 0)
    g_commander.legal_status = 0;
  else
    g_commander.legal_status = nls;
      
  increment_trumbles();    
  // calculate miss-jump (note: fuel already reduced at this point!)
  // TODO - fix the miss jump fuel thing so we don't get trapped
  g_my_stats.miss_jump = 0;
  // unarm missile if we have a lock
  if ( g_my_stats.missile_armed > 0) 
    unarm_missile();
  int r = get_rand();
  if (r > MISS_JUMP_CHANCE) {
    // 31C9

#ifndef HAVE_GBA_SHIPSETS
    ships = ship_collections[3];
    current_ship_flags = ship_flags[3];
#endif
    g_my_stats.miss_jump = 1;

    do {
      // add a few thargoids - originally this was hardcoded to 3...
      if (add_thargoids()==-1) {
        break;
      }
    } while ( g_object_types[SHIP_THARGOID] < (r&3) );

    g_commander.cpl_coord.y ^= 0x1e;
    g_star_count = WITCHSPACE_STAR_COUNT;
  } else {
    load_shipset();
    // set the planet and sun positions
    add_planet();
    // set planet location based on random seed
    int A = (cpl_seed.s1&3)+3;
    g_objects[0].location.z = A<<16;
    g_objects[0].location.x = g_objects[0].location.y = (A/2)<<16;

    // add random sun location too 
    add_new_ship(SUN);
    g_objects[1].location.z = (( (cpl_seed.s3&7)|1 )<<16);
    g_objects[1].location.z = -g_objects[1].location.z;

    g_objects[1].location.x = g_objects[1].location.y = ( cpl_seed.s5&3 )<<16;
    g_objects[1].rotx = g_objects[1].rotz = 0;
    g_star_count = DEFAULT_STAR_COUNT;
  }

  // oh and add new stars too
  g_new_stars = 0;
  // clear the number..
  // if we are on a chart screen, redraw it
  if (g_my_stats.miss_jump==0 && 
      (g_my_stats.screen == SCR_SHORT_CHART || g_my_stats.screen == SCR_LONG_CHART)) {
    manual_activate_screen(g_my_stats.screen + 1);
  } else  {
    // otherwise show front view..
    clear_text();
    if (g_my_stats.view != FRONT_VIEW || g_my_stats.screen != SCR_LAUNCHED) {
      set_front_view();
    }
    update_dials();
    dial(DIAL_FU, g_commander.fuel/2);
    play_sound(SOUNDFX_HYPERSPACE);
    launch_pattern(4,0);
  }
  if (g_my_stats.miss_jump)
    flight_message( IFF_FLIGHT_MESSAGE( DRIVE_MALFUNCTION ) );
  // reset hyperspace distance
  hyp1(0);
  g_my_stats.auto_pilot = 0;

  extern int g_panel_item;
  if (g_my_stats.screen == SCR_LAUNCHED && g_my_stats.miss_jump==0)
    g_panel_item = 5;  // select the JUMP drive thingy
  else if (g_my_stats.screen == SCR_LAUNCHED && g_my_stats.miss_jump)
    g_panel_item = 11; // witch space - select the chart
  else
    g_panel_item = 0;  // else select front view
}
    

void check_hyperspace(void) {
  // countdown counter = 0xf initially
  // then 5 afterwards
  g_hyperspace_timer--;
  if (g_hyperspace_timer == 0) {
    // print the number counter-1
    g_hyperspace_timer = 5;
    set_text_x(HYPERSPACE_COUNTER_X);
    g_yc = HYPERSPACE_COUNTER_Y;
    g_hyperspace_counter--;
    print_number(g_hyperspace_counter,2);
    if (g_hyperspace_counter==0) {
      // reached the end, do hyperspace 31F7
      g_commander.fuel -= hyp_info.distance;

      do_hyperspace();

    }
  }
}

int hyperspace_valid(void) {

  if (hyp_info.distance == 0) {
    return 0;
  }
  if (hyp_info.distance <= g_commander.fuel)
    return 1;
  
  return 0;
}

void start_hyperspace(void) {
  if (hyperspace_valid()) {
    g_hyperspace_timer = g_hyperspace_counter = HYPERSPACE_COUNTDOWN;
    set_text_x(HYPERSPACE_COUNTER_X);
    g_yc = HYPERSPACE_COUNTER_Y;
    print_number(g_hyperspace_counter,2);
    if (g_my_stats.screen == SCR_LAUNCHED) {
      // only show hyperspace on main views
      set_text_x(HYPERSPACE_INFO_X);
      g_yc = HYPERSPACE_INFO_Y;
      text2buffer(HYPERSPACE_STRING);
      text2buffer('-');
      text2buffer(' ');
      text2buffer(CH_PLANET_STRING);
    }
  }
}

void start_galactic_hyperspace() {

  g_commander.galaxy++;
  g_commander.galaxy&=7;

  // rotate the galaxy seed left
  unsigned char * seed = (unsigned char*)&g_commander.gal_seed;

  int i;
  for (i = 0; i < 6; i++) {
    int A = *seed;
    A *= 2;         // asl
    A |= (A>255);   // rol
    A&=0xff;        // 8 bit  
    *seed++=A;      // sta, inc x
    
  }
  
  hyp_coord.x = 0x60;
  hyp_coord.y = 0x60;
  // set front view
  if (g_my_stats.view != FRONT_VIEW)
    set_front_view();
  // find planet at location 
  hyp1(0); // 2F1F
  // set distance to 0
  hyp_info.distance = 0;

  // copy planet to cpl_coord
  g_commander.cpl_coord = hyp_coord;

  set_text_x(HYPERSPACE_INFO_X);
  g_yc = HYPERSPACE_INFO_Y;
  text2buffer(116); // galactic hyperspace
  g_hyperspace_timer = g_hyperspace_counter = HYPERSPACE_COUNTDOWN;
  set_text_x(HYPERSPACE_COUNTER_X);
  g_yc = HYPERSPACE_COUNTER_Y;
  print_number(g_hyperspace_counter,2);
}
    

// this is called every 32 frames
// check that we are close to the planet
void approach_planet(void) {
  unsigned long top =  abs(g_objects[0].location.x) | 
           abs(g_objects[0].location.y) | 
           abs(g_objects[0].location.z);
  if (top&0x7f0000) {

    return;
  }

  // 156A
  // calculate where the station would go
  vector32_t location = {};

  // the station location is effectively random,  
  // it is based on the planet's rotation matrix... 
  location.x = g_objects[0].location.x - (g_objects[0].rotmat[0].x*2);
  if (abs(location.x) >= 0xc000) {
    return;
  }
  location.y = g_objects[0].location.y - (g_objects[0].rotmat[0].y*2);
  if (abs(location.y) >= 0xc000) {
    return;
  }
  location.z = g_objects[0].location.z - (g_objects[0].rotmat[0].z*2);
  if (abs(location.z) >= 0xc000) {
    return;
  }

  // remove sun
  g_objects[1].type = 0;
  
  // ok, add
  add_station();

  g_objects[1].location = location;

  // the stations has the same rotmat as the planet, but 
  // by flipping the z axis !tada! it points towards the planet
  // note! Fix 28/03: the z and x axis of the planet are wrong? Elite 6502 
  // uses index 0 as the z axis, index 2 as the x axis. This has caused
  // me many a headache. This works, and is faithful to asm, but something is
  // not 100% correct. Grrrr.  Right, I'm off to bed.
  g_objects[1].rotmat[0] = g_objects[0].rotmat[2];
  g_objects[1].rotmat[1] = g_objects[0].rotmat[1];
  
  g_objects[1].rotmat[2].x = g_objects[0].rotmat[0].x;
  g_objects[1].rotmat[2].y = g_objects[0].rotmat[0].y;
  g_objects[1].rotmat[2].z = g_objects[0].rotmat[0].z;

}

