
#include <stdlib.h>
#ifdef USE_BIOS_DIVIDE
#include "bios_div.h"
#endif
#include "soup.h"
#include "planet.h"
#include "clipping.h"
#include "gbahandler.h"
#include "colours.h"
#include "commander.h"
#include "eliteloop.h"
#include "options.h"

// step size for curves
int g_stp = 0;

// 32 byte lut for atan
const unsigned char atan_tab[] = {
0x00, 0x01, 0x03, 0x04, 0x05, 0x06, 0x08, 0x09, 
0x0a, 0x0b, 0x0c, 0x0d, 0x0f, 0x10, 0x11, 0x12,
0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x19, 
0x1a, 0x1b, 0x1c, 0x1d, 0x1d, 0x1e, 0x1f, 0x1f
};
const unsigned char sne_tab[] = {
 0x00, 0x19, 0x32, 0x4a, 0x62, 0x79, 0x8e, 0xa2, 0xb5, 0xc6, 0xd5, 0xe2, 0xed, 0xf5, 0xfb, 0xff, 
 0xff, 0xff, 0xfb, 0xf5, 0xed, 0xe2, 0xd5, 0xc6, 0xb5, 0xa2, 0x8e, 0x79, 0x62, 0x4a, 0x32, 0x19
};


// returns ATAN(P/Q)
int atan6502(int P, int Q) {

  int top = abs(P);
  int bot = abs(Q);
  int sign = P^Q;
  int value = 0;
  if (top > bot) {
    top = abs(Q);
    bot = abs(P);
    if (bot) {
#ifdef USE_BIOS_DIVIDE  
      int A = bios_divide(top*256,bot);
#else
      int A = (top*256)/bot;
#endif
      value = 0x3f -  atan_tab[A/8];
    } else {
      value = 0x3f;
    }
  } else if (bot != 0) {
#ifdef USE_BIOS_DIVIDE  
    int A = bios_divide(top*256,bot);
#else
    int A = (top*256)/bot;
#endif
    value = atan_tab[A/8];
  } else {
    value = 0x3f;
  }

//  printf("value %d sign %d \n", value, sign);
  if (sign < 0) {
    value = (0x80-value);
  } 
  return value; 
}

void horizontal_line(int,int,int, int) LONG_CALL;
// draw a "sun" - a filled circle with wobbly lines if big
void sun(int x, int y, int size, int smooth) {

  // CHKON 3C28
  int lhs = x-size;  // lhs of circle
  int rhs = x+size;

  if ( (rhs < CLIP_X_MIN) || ( lhs > CLIP_X_MAX) )
    return;

  int top = y - size;  // top of circle
  int bot = y + size;
  if ( (bot < CLIP_Y_MIN) || ( top > CLIP_Y_MAX) )
    return;

  if (top < CLIP_Y_MIN) {
    top = CLIP_Y_MIN;
  }
  if (bot > CLIP_Y_MAX) {
    bot = CLIP_Y_MAX;
  }

  int rand_mask = (size>0x60)*4 + (size>0x28)*2 + (size>0x10);
  if (smooth)
    rand_mask = 0;

  // sun could be sped up by using Bresenham circle
  // but this runs ok on hw, so I'm tempted to leave it as is
  int ypos;
  // create a block of 8 pixels
  unsigned long pixel8 = lineColour&0xf;
  pixel8 |= pixel8<<4;
  pixel8 |= pixel8<<8;
  pixel8 |= pixel8<<16;

  for (ypos = top; ypos < bot; ypos++) {
    int w = (size*size) - ((ypos-y)*(ypos-y));
    w = bios_sqrt(w);
    w+= get_rand()&rand_mask;
    int x1 = x-w;
    if (x1 < CLIP_X_MIN) x1 = CLIP_X_MIN;
    if (x1 > CLIP_X_MAX) x1 = CLIP_X_MAX;
    int x2 = x+w;
    if (x2 < CLIP_X_MIN) x2 = CLIP_X_MIN;
    if (x2 > CLIP_X_MAX) x2 = CLIP_X_MAX;

    if (x1 != x2) {
      horizontal_line(x1,x2,ypos,pixel8);
    }
  } 

}

// no checks, step size not set
void circle2(int x, int y, int size) {

  // set count and flag values
  int cnt = 0;
  
  coord_t p1 ={};
  coord_t p2 ={0xff,0xff};
  int pos = 0;
  for (cnt = 0; cnt < 0x41; cnt += g_stp) {
    // calc sine wave size
    // x = centrex + sin(cnt&31)*size
    int K = (sne_tab[cnt & 31] * size)/256;
    if (cnt >= 0x21) {
      K ^= -1;
      K++;
    }
    int x1 = K + x;
    // y = centrey + sin(cnt+16 & 31) * size
    K = (sne_tab[(cnt+16) & 31] * size)/256;
    // next quadrant
    if (((cnt + 0xf)&0x3f) >= 0x21) {
      K ^= -1;
      K++;
    }
    int y1 = K + y;
    if (!pos) {
      pos ^= 1;
      p1.x = x1;
      p1.y = y1;
      if (p2.y != 0xff) {
        // already drawn a line
        draw_clipped_line(&p1, &p2);
      }
    } else {
      p2.x = x1;
      p2.y = y1;
      pos ^= 1;
      draw_clipped_line(&p1, &p2);
    }
  }
}

// returns 1 for drew, 0 for failed
int circle(int x, int y, int size) {
  // first check circle -
  //  x + size < lhs of screen fails
  //  x - size > rhs of screen fails 
  //  y + size < top of screen fails
  //  y - size > bot of screen fails

  // CHKON 3C28
  int lhs = x-size;  // lhs of circle
  int rhs = x+size;

  if ( (rhs < CLIP_X_MIN) || ( lhs > CLIP_X_MAX) )
    return 0;
  
  int top = y - size;  // top of circle
  int bot = y + size;
  if ( (bot < CLIP_X_MIN) || ( top > CLIP_Y_MAX) )
    return 0;

  // else really draw it

  // calc step size
  g_stp = 8;
  if (size >= 8) {
    g_stp /= 2;
  }
  if (size >= 0x3c) {
    g_stp /= 2;
  }
  if (g_game_options.planet_solid && 
      g_my_stats.screen == SCR_LAUNCHED)
    sun(x,y,size,1);
  else
    circle2(x,y,size);
  return 1;
}

void draw_curve(int x1, int y1, int x2, int y2, int angle, int cx , int cy,
    int target) {

  // x1y1 is ZERO axis scaled by far awayness
  // x2y2 is ONE axis scaled by far awayness
  // angle is z portion of angle between the two axes
  
  coord_t p1 ={};
  coord_t p2 ={0xff,0xff};
  int pos = 0;
  int i;
  for (i = 0; i < target; i+=g_stp) {
    
    // draw only front half of curve
    int sn = sne_tab[angle&0x1f];  // sine of z portion
    int x = (sn*x1)/256;
    int y = (sn*y1)/256;
    if ( angle < 0x21) {
      x = -x;
      y = -y;
    } 
    sn = sne_tab[(angle+16)&0x1f];
    if ( ((angle+0xf)&0x3f) < 0x21) {
      x = x - ((sn*x2)/256) ;
      y = y - ((sn*y2)/256);
    } else {
      x = x + ((sn*x2)/256);
      y = y + ((sn*y2)/256);
    }
    x += cx;
    y  =  cy - y;

    if (!pos) {
      pos ^= 1;
      p1.x = x;
      p1.y = y;
      if (p2.y != 0xff) {
        // already drawn a line
        draw_clipped_line(&p1, &p2);
      }
    } else {
      p2.x = x;
      p2.y = y;
      pos ^= 1;
      draw_clipped_line(&p1, &p2);
    }
    angle = (angle + g_stp)&0x3f;
  }
}

#define ZERO 0
#define ONE 1
#define TWO 2


void beachball(int x, int y, int size, stellarobject_t * planet) { // 3884
  // uses rotation matrix
  int az = atan6502(-planet->rotmat[ZERO].z, planet->rotmat[ONE].z);
  // tan (ang) = op/adj
  // angle between zero and one
  if (planet->rotmat[ZERO].z >=0)
    az ^= 0x80;
  az /= 4;

#define DIVIDEND  245
#ifdef USE_BIOS_DIVIDE
  int planetz = bios_divide(planet->location.z, DIVIDEND);
#else
  int planetz = (planet->location.z/DIVIDEND);
#endif
  
#ifdef USE_BIOS_DIVIDE
  int x1 = bios_divide(planet->rotmat[ZERO].x,planetz); 
  int y1 = bios_divide(planet->rotmat[ZERO].y,planetz); 
  // X axis scaled by distance
  int x2 = bios_divide(planet->rotmat[ONE].x ,planetz); 
  int y2 = bios_divide(planet->rotmat[ONE].y ,planetz); 
#else
  int x1 = (planet->rotmat[ZERO].x)/planetz; 
  int y1 = (planet->rotmat[ZERO].y)/planetz; 
  // X axis scaled by distance
  int x2 = (planet->rotmat[ONE].x )/planetz; 
  int y2 = (planet->rotmat[ONE].y )/planetz; 
#endif
  // Y axis scaled by distance

  //printf("curve %d %d to %d %d about %d %d with %d\n", x1,y1,x2,y2,x,y,az);
  draw_curve(x2,y2,x1,y1,az,x,y,0x1f);
  
  az = atan6502(-planet->rotmat[ZERO].z, planet->rotmat[TWO].z);
  if (planet->rotmat[ZERO].z >= 0)
   az ^= 0x80;
  az /= 4;
  
#ifdef USE_BIOS_DIVIDE
  x2 = bios_divide(planet->rotmat[TWO].x, planetz); 
  y2 = bios_divide(planet->rotmat[TWO].y, planetz); 
#else
  x2 = ((short)planet->rotmat[TWO].x )/planetz; 
  y2 = ((short)planet->rotmat[TWO].y )/planetz; 
#endif

  draw_curve(x2,y2,x1,y1,az,x,y,0x1f);
}

// crtr starts at 38C0 
void crater(int x, int y, int size, stellarobject_t * planet) {
  if (planet->rotmat[ONE].z < 0) {
    // crater out of view
    return;
  }
#ifdef USE_BIOS_DIVIDE
  int AP = bios_divide(planet->rotmat[ONE].x*256, planet->location.z);
#else
  int AP = (planet->rotmat[ONE].x*256) / planet->location.z;
#endif
  AP = AP * 0xDE;  // size of crater
  int crtcenx = x+(AP/256);
#ifdef USE_BIOS_DIVIDE
  AP = bios_divide((planet->rotmat[ONE].y*256) , planet->location.z);
#else
  AP = (planet->rotmat[ONE].y*256) / planet->location.z;
#endif
  AP = AP * 0xDE;  // size of crater
  int crtceny = y-(AP/256);
  // 3915
#ifdef USE_BIOS_DIVIDE
  int x1 = bios_divide(planet->rotmat[ZERO].x*256, planet->location.z);  x1 /= 2;
  int y1 = bios_divide(planet->rotmat[ZERO].y*256, planet->location.z);  y1 /= 2;
  int x2 = bios_divide(planet->rotmat[TWO].x*256, planet->location.z);  x2 /= 2;
  int y2 = bios_divide(planet->rotmat[TWO].y*256, planet->location.z);  y2 /= 2;
#else
  int x1 = (planet->rotmat[ZERO].x*256)/planet->location.z;  x1 /= 2;
  int y1 = (planet->rotmat[ZERO].y*256)/planet->location.z;  y1 /= 2;
  int x2 = (planet->rotmat[TWO].x*256)/planet->location.z;  x2 /= 2;
  int y2 = (planet->rotmat[TWO].y*256)/planet->location.z;  y2 /= 2;
#endif 
  draw_curve(x1,y1,x2,y2, 0 , crtcenx, crtceny,0x41);

  
}


// 3845
void draw_planet(stellarobject_t * planet) {
  static int drawn = 0;
  if (planet->location.z < 0 || planet->location.z > 0x300000 
      || planet->location.z < 0xff || drawn) {
    return;
  }
  // first project the coordinates (overflow here?)
#ifdef USE_BIOS_DIVIDE
  int x =bios_divide(planet->location.x*256, planet->location.z);
  int y =bios_divide(planet->location.y*256, planet->location.z);
#else
  int x = (planet->location.x*256) / planet->location.z;
  int y = (planet->location.y*256) / planet->location.z;
#endif

  int cx = x + SCREEN_SIZE_X;
  int cy = SCREEN_SIZE_Y - y;
  // so we have x/z + screenx and y/z + screeny
  // then "project" 0x60*256 / Z (circle size)
#ifdef USE_BIOS_DIVIDE
  int size = bios_divide((0x600000) , planet->location.z);
#else
  int size = (0x600000) / planet->location.z;
#endif
  // then (wipe the old lines and) draw the circle 
  // if 0x6000/Z > 0xff then return  (too close to draw?)
  if (size > 0xf8) // fix disappearing sun bug?
    size = 0xf8;
  lineColour = PLANET_COL;
  if (planet->type != SUN) {
    // if the circle wasn't drawn for some reason, return
    if (circle(cx,cy,size) && size > 6) {

      if (g_game_options.planet_solid)
        lineColour = 0;
      if (planet->type == CRATER) {
        crater(cx,cy,size,planet);
      } else {
        // we drew it - now the beach ball effect
        beachball(cx, cy, size,  planet); // 3884
      }
    }
  } else {
    // 39FE 
    lineColour = SUN_COL;
    sun(cx,cy,size,0);
  }
}

