#include <stdlib.h>
#ifdef USE_BIOS_DIVIDE
#include "bios_div.h"
#endif
#include "scanner.h"
#include "gfx.h"
#include "gbahandler.h"
#include "motion.h"
#include "commander.h"
#include "tactics.h"


#define ZOOM_FACTOR 2
#define SCAN_RANGE  0x4000
#define SCAN_RANGE_SQ 0x1600

int g_compass_index = 0;

void reset_scanner_options()
{
  // make compass point to planet
  g_compass_index = 0;
  // reset zoom
  g_my_stats.scan_zoom = 0;
  
}

void flip_compass(void) {
  g_compass_index ^= 1;
}

int can_flip_compass(void) {
  return (g_object_types[2] == 0  && g_my_stats.miss_jump == 0/*not in witch space...*/);
}

#ifdef HAVE_BBC_COMPASS
void update_compass(void) {
  // compass routine based on 35F7 
  int index = 1;
  if (g_object_types[2]==0) {
    index = g_compass_index;
  }

  // normalise the location of object [index]
  // 4283
  int x = g_objects[index].location.x;
  int y = g_objects[index].location.y;
  int z = g_objects[index].location.z;

  int total = abs(x)|abs(y)|abs(z);
  
  while (total < 0x10000) {
    total*=2;
    x *= 2;
    y *= 2;
    z *= 2;
  }

  vector32_t v = { (x/256), (y/256), (z/256)};
  norm(&v);
  
  // sort out individual x, y, z
#ifdef USE_BIOS_DIVIDE
  x = bios_divide(v.x, 0x14);
  y = bios_divide(v.y, 0x14);
#else
  x = (v.x)/0x14;
  y = (v.y)/0x14;
#endif

  move_compass_sprite(x/256,y/256,v.z);
}
#else
// compass code based on speccy version
void update_compass(void) {
  int index = 1;
  if (g_object_types[2]==0) {
    index = g_compass_index;
  }
  
  int x = g_objects[index].location.x;
  int y = g_objects[index].location.y;
  int z = g_objects[index].location.z;

  int distance = 0;
  // check overflow of s32
  int overflow = (abs(x) > 0xb504 ||
      abs(y) > 0xb504 ||
      abs(z) > 0xb504); /*overflow guaranteed: b505*b505 > 0x80000000 */
  if (!overflow) {
    distance = x*x;
    distance += y*y;
    overflow = (distance < 0);
    if (!overflow) {
      distance += z*z;
      overflow = (distance < 0);
    }
  }

  if (overflow) {
    // use obj far away code
    distance = ((x/256)*(x/256)) +
               ((y/256)*(y/256))+
               ((z/256)*(z/256));
  } else {
    // use close code
    // distance = x*x + y*y +z*z; /*already is this*/
    x *= 256;
    y *= 256;

  }
  // find projected position:
  distance = bios_sqrt(distance);
  if (distance == 0)
    distance = 0xFFFF;
#ifdef USE_BIOS_DIVIDE
  int px = bios_divide(x , distance);
  int py = bios_divide(y , distance);
#else
  int px = (x / distance);
  int py = (y / distance);
#endif
  //cx, cy rng 0-7
  move_compass_sprite(px/32,py/32,z);
}
#endif 


void scanner(int index) {
  // update object on scanner
  // new scanner lives at $5544
  // invisible or planet
  stellarobject_t * obj = &g_objects[index];
  if (/*(obj->missiles & FLAG_INVISIBLE) || */(obj->type <= 0)) { 
    return;
  }
#ifdef HAVE_BBC_SCANNER
  int d = abs(obj->location.x) | abs(obj->location.y) | abs(obj->location.z);
  if (d&0xc000) {
    return; // far away
  }
  int x = obj->location.x / 256;
  int y = obj->location.y / 512;
  int z = obj->location.z / 1024;
#else
  // modified scanner code for GBA (wider, flatter, with 2x zoom)
  int factor = g_my_stats.scan_zoom;
  int limit = factor==0?SCAN_RANGE:(SCAN_RANGE/ZOOM_FACTOR);
  if (abs(obj->location.x) > limit ||
      abs(obj->location.y) > limit ||
      abs(obj->location.z) > limit )
  {
    return;
  }
  else
  {
    int x = obj->location.x / 256;
    int z = obj->location.z / 256;
    int distsq = x*x + z*z;
    limit = factor==0?SCAN_RANGE_SQ:(SCAN_RANGE_SQ/(ZOOM_FACTOR*ZOOM_FACTOR));
    if (distsq > limit){
      return;
    }
  }
  int x,y,z;
  if (factor==0) {
    x = (3 * (obj->location.x) / 4)  / 256;
    y = (5 * (obj->location.y) / 16) / 256;
    z = (5 * (obj->location.z) / 16) / 256;
  } else {
    x = (3 * (obj->location.x) / (4/ZOOM_FACTOR))  / 256;
    y = (5 * (obj->location.y) / (16/ZOOM_FACTOR)) / 256;
    z = (5 * (obj->location.z) / (16/ZOOM_FACTOR)) / 256;
  }
#endif

  // colour, unless IFF not fitted
  int colour = COLOUR_FRIENDLY;
  if (g_commander.equipment[IFF_SYSTEM]) {

    /*
    ; X = 4  if type >= 8   col1=ff   col2=f0     11 10
    ; X = 3  if type == 1   col1=f0   col2=ff     10 11  missile
    ; X = 2  if type < 8    col1=ff   col2=0f     11 01  junk
    ; X = 1  bit 6 set      col1=00   col2=f0     00 10  escape pod?
    ; X = 0  bit 5 set      col1=00   col2=00     00 00  friendly
    ; X = 0  no IFF         col1=00   col2=00     00 00
    */
    if (obj->flag & FLAG_POLICEMAN) {
      colour = COLOUR_FRIENDLY;
    } else if (obj->flag&FLAG_DEFENCELESS) {
      colour = COLOUR_OFFICIAL;
    } else if (obj->type == SHIP_MISSILE) {
      colour = COLOUR_MISSILE;
    } else if (obj->type <= SHIP_SPLINTER) {
      colour = COLOUR_JUNK;
    } else {
      colour = COLOUR_PIRATE;
    }

  }
  if (abs(x) < 70 /*&& (abs(y+z)) < 52 */) 
  {
    lollipop(x, y, z, LOLLIPOP_COL(colour), index);
  }
}

