#include <stdlib.h>
#include "script.h"
#include "gba.h"
#include "gbahandler.h"
#include "shipdata.h"
#include "motion.h"
#include "universe.h"
#include "tactics.h"
#include "autopilot.h"
#include "soundfx.h"
#include "clipping.h"
#include "colours.h"

#define END_ENTRY     0
#define ADD_SHIP      1
#define SET_POSITION  2
#define MOVE_SHIP     3
#define FIRE          4

#define SCRIPT_COUNT 3

static const s16 script0[] = {
  ADD_SHIP, SHIP_WORM,      // not a worm, a sidewinder
  SET_POSITION, 0, 100, 0,    // start above us in the centre
  MOVE_SHIP, 0 , 52, 802,  // move to where the logo is, more or less
  FIRE,  // move to where the logo is, more or less
  MOVE_SHIP, 0 , 82, 902,  // move to where the logo is, more or less
  MOVE_SHIP, 30, 95, 1000, // fly away from us and upwards
  END_ENTRY
};

static const s16 script1[] = {
  ADD_SHIP, SHIP_KRAIT,
  SET_POSITION, 100, 100,  -210,    // start above us and on the right side
  MOVE_SHIP,   50,  60, 320,    // fly central ish
  MOVE_SHIP,  400,100, 1200,    // then fly back out to the right
  END_ENTRY
};


static const s16 script2[] = {
  ADD_SHIP, SHIP_ADDER,
  SET_POSITION, -100, 100,  -420,    // start under us and on the left side
  MOVE_SHIP,   -50,  -10, 220,    // fly central ish
  MOVE_SHIP,  -400, -300, 1200,    // then fly back out to the right
  END_ENTRY
};

static const s16 * scripts[SCRIPT_COUNT] = {
  script0,
  script1,
  script2
};

int g_script_index[SCRIPT_COUNT];

// this is where the ships should head to
vector32_t g_script_targets[SCRIPT_COUNT] __attribute__ ((section (".sbss")));

static const coord_t laser_end = {SCREEN_SIZE_X,SCREEN_SIZE_Y};
// update the scripts for each ship
int update_script(int * pIndex, int s) {
  int index = *pIndex;
  
  int have_ended = 0;
  switch (scripts[s][index++]) {
    case END_ENTRY :
      {
        g_objects[s+2].ai_state = 0xcf;
        g_objects[s+2].flag = FLAG_ANGRY;
        have_ended = 1;
      }
      break;
      
    case ADD_SHIP:
      {
        s16 id = scripts[s][index++];
        // add ship id - and store the id somewhere (!!)
        //int shipid = 
        add_new_ship(id);
        // while I could do this with another array and allocating and stuff...
        // why bother? I know I have a sun and a planet, otherwise I can't fly about
        //assert(shipid == (s+2));
        g_objects[2+s].rotmat[2].z = 0x6000;
      }
      break;
    case SET_POSITION:
      {
        // jump ship 's' to position to x, y, z
        g_objects[s+2].location.x = scripts[s][index++];
        g_objects[s+2].location.y = scripts[s][index++];
        g_objects[s+2].location.z = scripts[s][index++];
        /*gbaprintf("SET_POSITION to %d %d %d\n", 
            g_objects[2+s].location.x,
            g_objects[2+s].location.y,
            g_objects[2+s].location.z);*/
      }
      break;
    case MOVE_SHIP:
      {
        g_script_targets[s].x = scripts[s][index++];
        g_script_targets[s].y = scripts[s][index++];
        g_script_targets[s].z = scripts[s][index++];
        // fly ship to x, y, z using autopilot 
        /*gbaprintf("MOVE_SHIP to %d %d %d\n", 
            g_script_targets[s].x,
            g_script_targets[s].y,
            g_script_targets[s].z);*/
        
      }
      break;
    case FIRE:
      {
        play_sound(SOUNDFX_ENEMY_FIRE);
        g_objects[NUM_SHIPS-1].missiles = 0x80;
        g_objects[NUM_SHIPS-1].type = SHIP_ASTEROID;
        // draw a laser bolt and an explosion sound?
        play_sound(SOUNDFX_EXPLOSION);
        entry(&g_objects[2], ships[g_objects[2].type-1]);
        lineColour = VECTOR_COL;
        draw_clipped_line( 
            &g_scrncrd[ships[g_objects[2].type-1]->laser_vertex/4],
            &laser_end);
      }
      break;
    default:
      have_ended = 1;
      break;
  }

  *pIndex = index;
  return have_ended;
}

int script_fly_ship(int index) {
  // fly ship id to its waypoint
  stellarobject_t* obj = &g_objects[index+2];

  
  int x= g_script_targets[index].x;
  int y= g_script_targets[index].y;
  int z= g_script_targets[index].z;
  // see how close they are
  int diff = abs(obj->location.x - x) + 
    abs(obj->location.y-y) + 
    abs(obj->location.z - z);
  if (diff < 128) {
    // reached target
    //gbaprintf("Ship %d reached %d %d %d!!\n",index,x,y,z);
    return 1;
  }

  vector32_t vship;
  norm_big(&vship,obj->location.x - x,obj->location.y - y,obj->location.z - z);

  //int attack_ang =((obj->rotmat[2].x/256) * vship.x) +
  //  ((obj->rotmat[2].y/256) * vship.y) +
  //  ((obj->rotmat[2].z/256) * vship.z);
  
  //attack_ang = -attack_ang;
  flip_vect(&vship);
  fly_to_vector(obj,&vship);
  obj->accel = obj->accel?1:0;
  
  //fly_to_vector_final(obj, &vship, attack_ang, 4);
  return 0;
}

void initialise_scripted_ships() 
{
  int i;
  for (i = 0; i < SCRIPT_COUNT; i++) {
    g_script_index[i] = 0;
    // hope the first 2 are start and set position...
    int ended = update_script(&g_script_index[i], i);
    //assert(ended==0);
    ended = update_script(&g_script_index[i], i);
    //assert(ended==0);
    // the next is the first way point
    ended = update_script(&g_script_index[i], i);
    //assert(ended==0);
  }
}

int fly_script_ships() 
{
  int i;
  int ended_count = 0;
  for (i = 0; i < SCRIPT_COUNT; i++) 
  {
    if (g_script_index[i] != -1) {
      // fly the 3 ships
      int reached_target = script_fly_ship(i);
      if (reached_target) {
        int ended = update_script(&g_script_index[i], i);
        if (ended) {
          // that's all folks!
          g_script_index[i] = -1;
          ended_count++;
        }
      }
    } else {
      ended_count++;
    }
    mveit(&g_objects[i+2], ships[g_objects[i+2].type-1], i+2);
    entry(&g_objects[i+2], ships[g_objects[i+2].type-1]);
    
  }
  // tell the caller when all ships have reached their destinations
  return (ended_count == SCRIPT_COUNT);
}

