#include "screenmode.h"
#include "gbahandler.h"
#include "keypad.h"
#include "gfx.h"
#include "motion.h"
#include "clipping.h"
#include "colours.h"
#include "stars.h"

extern unsigned char letter_data[];
extern unsigned char letter_vertices_x[];
extern unsigned char letter_vertices_y[];

#define ROW_COUNT 4
#define LINE_COUNT 84

static int letter_coord_index[ROW_COUNT];
typedef struct {
  coord_t start, end;
} letter_line_t;

// we have several rows of letters
static letter_line_t letter_rows[ROW_COUNT][LINE_COUNT];  // 16*5 = 80!

// uses letters..
static void add_letter(char c, int x_offset, int y_offset, int row) {

  // each letter needs to be unpacked into letter_coord

  if (c >= ',' && c <= '~') {
    const unsigned char * data_ptr =  &letter_data[(c-',')*5];
    // 5 lines per char
    int i;
    for (i = 0; i < 5; i++) {
      if (!(*data_ptr)) {
        break;
      }
      int index = (*data_ptr)&0xf;
      letter_rows[row][letter_coord_index[row]].start.x = x_offset + letter_vertices_x[index];
      letter_rows[row][letter_coord_index[row]].start.y = y_offset - letter_vertices_y[index];
      index = (*data_ptr)>>4;
      letter_rows[row][letter_coord_index[row]].end.x = x_offset + letter_vertices_x[index];
      letter_rows[row][letter_coord_index[row]].end.y = y_offset - letter_vertices_y[index];
      letter_coord_index[row]++;
      if (letter_coord_index[row] == LINE_COUNT) {
        letter_coord_index[row]--;
        return;
      }
      data_ptr++;
    }
  }
}

static void init_scroll_text(const char * text) {
  int x_offset = 0;
  int y_offset = CLIP_Y_MAX;
  int row = 0;

  for (row = 0; row < ROW_COUNT; row++) {
    letter_coord_index[row] = 0;
    int i;
    for (i = 0; i < 64; i++) {
      letter_rows[row][i].start.y = 0;
    }
  }

  row = 0;
  while (*text) {
    add_letter(*text,x_offset, y_offset, row);
    x_offset += 16;
    if (x_offset >= 256) {
      x_offset = 0;
      letter_rows[row][letter_coord_index[row]++].start.y = 0;
      row++;
      //y_offset -= 30;
    }
    text++;
  }
  letter_rows[row][letter_coord_index[row]++].start.y = 0;
}

static int draw_scroll_text(int scroll, int row) {
  int i;
  int row_offset = row*30;
  if ( ( scroll + row*16 ) < 8) 
    return 0;
  for (i = 0; i < letter_coord_index[row]; i++) {
    if (letter_rows[row][i].start.y == 0) {
      break;
    }
    // draw the letter here...
    coord_t start, end;
    int y = (letter_rows[row][i].start.y - scroll)-row_offset;
    if (y < 0)
      continue;
    int z = y*4+128;

    start.x = ((letter_rows[row][i].start.x-SCREEN_SIZE_X)*256)/z;
    start.x += SCREEN_SIZE_X;
    start.y = ((0x50+scroll-y)*256)/z;
    start.y += 0x40;

    y = (letter_rows[row][i].end.y - scroll)-row_offset;
    if (y < 0)
      continue;
    z = y*4+128;
    end.x = ((letter_rows[row][i].end.x-SCREEN_SIZE_X)*256)/z;
    end.x += SCREEN_SIZE_X;
    end.y = ((0x50+scroll-y)*256)/z;
    end.y += 0x40;

    draw_clipped_line( &start, &end);
  }
  return 1;
}
void do_scroll_text(const char * text) {
  init_scroll_text(text);
  int scroll = (3*CLIP_Y_MAX*256)/4;
  int draw = 0;
  //while (scroll >= 0) 
  for(;;)
  {
    lineColour = SUN_COL;
    int row;
    for (row = 0; row < ROW_COUNT; row++) {
      draw = draw_scroll_text(scroll/256,row);
    }
    if (draw == 0) 
      break;
    scroll -= 100;
    wait_vsync_int();
    swap_buffer();
    clear_screen();
    update_stars(0);
    UpdateKeys();
  }
}

