#include "upgrade.h"
#include "menu.h"
#include "gbahandler.h"
#include "gfx.h"
#include "text.h"
#include "soup.h"
#include "encyclo.h"
#include "eliteloop.h"
#include "dialgauge.h"

extern int g_current_ship_submenu;

int cancel_showroom(int index) 
{
  clear_screen();
  swap_buffer();
  clear_screen();
  draw_border();
  swap_buffer();
  clear_text();
  buy_ship();
  set_menu_selection(g_current_ship_submenu);
  return 0;
}

// map the purchase index to the encyclopedia index
static const unsigned char shipindex_to_encyclo[] = 
{
  0,  // ADDER
  12, // GECKO
  18, // MORAY    
  6,  // COBRA MK1
  14, // IGUANA   
  19, // OPHIDIAN 
  5,  // CHAMELEON
  7,  // COBRA MK3
  11, // F\x90""-D
  13, // GHAVIAL  
  17, // MONITOR  
  20, // PYTHON   
  3,  // BOA      
  2,  // ASP MK2    
  1    // ANACONDA 
};

static const unsigned char relative_equipment_cost[] = {
  8,
  2,
  7,
  4,
  0
};

#define ARMAMENTS 0x23
#define SPEED 0x24
#define CARGO 259
#define HULL      0x2c
#define FUEL_TEXT   0x69
#define MISSILES_TEXT   0x6A
#define ENERGY_UNIT_TEXT   0x72
#define ECONOMY        0xc2
#define FACTOR        0x2f

#define SHOWROOM_X   2
#define SHOWROOM_DATA_X   (SHOWROOM_X+9)
#define SHOWROOM_Y   2

void display_showroom(int id) 
{
  // for the given ship, display its information...
  /*
  laser_strength[4]; // use index 0, as it varies the most : 0xe - 0x1c
  laser_mounts;   // 1-4
  missile_count;  // 2-17
  hull_strength;  // 0x4-0xd
  regen_rate;     // 1-2
  max_speed;      // 0x15 - 0x3c
  total_cargo;    // 0x7 - 0xfe (!!)
  max_fuel;       // 60-125
  equipment_cost; // 0 - 4  (though 1 is cheapest)
  turning_circle; // 0x7f - 0xaf^0xff to 0x7f   2f to 7f  (work out this - af to get a rough idea)
  */
  clear_text();
#if 0

  g_yc = SHOWROOM_Y;
  set_text_x(SHOWROOM_X);
  mission2buffer(9);
#endif
#if 1
  setup_dial_gauges();
  const player_ship_t * ship_data = &player_ship_data[id];

  // armaments /////////////////////
  g_yc = SHOWROOM_Y;
  set_text_x(SHOWROOM_X);
  encyclo2buffer(ARMAMENTS);
  showroom_dial(ship_data->laser_strength[0]-0xd, 0x1c-0xd, g_xc, g_yc,3,2);
  // then say  "front, rear, left,right" underneath
  g_yc++;
  set_text_x(SHOWROOM_X+10);
  int mounts = ship_data->laser_mounts;
  int i;
  for (i = 0; i < mounts; i++) {
    textmod = NAME_CASE;
    text2buffer(0x60 + i);  // front, rear, left, right
    if ( (mounts-1) != i) 
      text2buffer(','); 

  }
  
  // missiles /////////////////////
  g_yc++;
  set_text_x(SHOWROOM_X);
  textmod = NAME_CASE;
  text2buffer(MISSILES_TEXT);
  print_dots('S');
  set_text_x(SHOWROOM_X+10);
  showroom_dial(ship_data->missile_count-1, 0x11, g_xc, g_yc,3,2);
  //print_number(ship_data->missile_count, 2);


  // hull /////////////////////
  g_yc += 2;
  set_text_x(SHOWROOM_X);
  encyclo2buffer(HULL);
  set_text_x(SHOWROOM_X+10);
  showroom_dial(ship_data->hull_strength-0x4, 0xd-0x4, g_xc, g_yc,3,2);

  
  // regen rate  /////////////////////
  g_yc += 2;
  set_text_x(SHOWROOM_X);
  textmod = NAME_CASE;
  print_dots(ENERGY_UNIT_TEXT);
  set_text_x(SHOWROOM_X+10);
  showroom_dial(ship_data->regen_rate, 0x2, g_xc, g_yc,3,2);

  
  // speed  /////////////////////
  g_yc += 2;
  set_text_x(SHOWROOM_X);
  encyclo2buffer(SPEED);
  set_text_x(SHOWROOM_X+10);
  showroom_dial(ship_data->max_speed/4, 15, g_xc, g_yc,3,2);
  
  // cargo  /////////////////////
  g_yc += 2;
  set_text_x(SHOWROOM_X);
  soup2buffer(CARGO);
  text2buffer(':');
  set_text_x(SHOWROOM_X+10);
  // there are rally 4 "levels" of cargo carrying ships, change colours to compare
  if (ship_data->total_cargo < 0x20)
    showroom_dial(ship_data->total_cargo/2, 15, g_xc, g_yc,3,2);
  else if (ship_data->total_cargo < 0x40)
    showroom_dial(ship_data->total_cargo/4, 15, g_xc, g_yc,4,3);
  else if (ship_data->total_cargo < 0x80)
    showroom_dial(ship_data->total_cargo/8, 15, g_xc, g_yc,5,4);
  else
    showroom_dial(ship_data->total_cargo/16, 15, g_xc, g_yc,6,5);
  
  // fuel  /////////////////////
  g_yc += 2;
  set_text_x(SHOWROOM_X);
  print_dots(FUEL_TEXT);
  set_text_x(SHOWROOM_X+10);
  showroom_dial(ship_data->max_fuel/8, 15, g_xc, g_yc,3,2);
  // economy of equipment /////////////////////
  g_yc += 2;
  textmod = NAME_CASE;
  set_text_x(SHOWROOM_X);
  print_dots(ECONOMY);
  set_text_x(SHOWROOM_X+10);
  showroom_dial(8 - relative_equipment_cost[ship_data->equipment_cost], 8, g_xc, g_yc,3,2);
  
  // maneuvreability  /////////////////////
  g_yc += 2;
  set_text_x(SHOWROOM_X);
  soup2buffer(258);
  text2buffer(':');
  set_text_x(SHOWROOM_X+10);
  showroom_dial( (ship_data->turning_circle-0xaf)/8, 10, g_xc, g_yc,3,2);
  
  reset_menu(1, 1);
  set_menu_callbacks(0,0,cancel_showroom,cancel_showroom);
  setup_encyclo_ship(shipindex_to_encyclo[id]);
  g_objects[0].location.z = 0x1000;
#endif
}


  
