/*
 * Planet description string generation
 * On the planet information screen there is a
 * description of each planet, e.g.
 * 
 * The planet Ribilebi is most famous for its 
 * vast oceans and Its Fabulous Goat Soup.
 *
 * Not as "clean" as C.Pinder's work, as it is 
 * closer to how the 6502 code works...
 */


#include "keypad.h"
#include "screenmode.h"
#include "encyclo.h"
#include "eliteloop.h"
#include "commander.h"
#include "navigation.h"
#include "galaxy.h"
#include "text.h"
#include "soup.h"
#include "input.h"
#include "gfx.h"
void dictionary2(int word);

extern const unsigned char * const mission_data[];

int g_have_icon = 0;
rand_t g_rand_seed = { 0x02, 0x19, 0x02, 0x19};

// an explantion is needed here:
// the first "seed.r0*2" is really a ROL, bit 0 is set to the carry flag
// In most cases the carry is clear, so *2 does the same job
// but when we loop over 3 new random numbers (entry == 18), the code is 
//    JSR get_rand
//    .... & 3 etc ...
//    TAY            ; Y = random&3
// loop:
//    JSR get_rand
//    .... & 3e, etc ...
//    DEY 
//    BPL loop
// here, DEY leaves the carry flag set (decrement doesn't underflow Y) as well
// as unsetting the negative flag, while in the loop.
// An example of this difference in carry flag "emulation":
// in Speccy (Z80) Elite, planet Leleer is plagued by a vicious disease 
// meanwhile the 6502 (BBC,NES,C64) Leleer is plagued by deadly earthquakes.
// mission descriptions may be different too?
int get_rand_carry(int *carry) {
  int A = (g_rand_seed.r0*2)|(*carry!=0);
  int X = A&0xff;
  A = (X + g_rand_seed.r2 + (A>0xff)); // carry from this used below
  g_rand_seed.r0 = A;
  g_rand_seed.r2 = X;
  A = (g_rand_seed.r1 + g_rand_seed.r3 + (A>0xff));
  *carry = (A>0xff);
  A&=0xff;
  X = g_rand_seed.r1;
  g_rand_seed.r1 = A;
  g_rand_seed.r3 = X;
  return A;
}
int get_rand(void) {
  // $3332 - and don't care about carry flag
  int carry = 0;
  return get_rand_carry(&carry);
}
int get_2rand(int * A, int * X) {
  int carry = 0;
  *A = get_rand_carry(&carry);
  *X = g_rand_seed.r3;
  return carry;
}


// not really "random", but contains other dictionary entries
// that are indexed based on the planet random seed
unsigned char random_table[] = {
  0x00, // for Z offset
  0x10, 0x15, 0x1a, 0x1f, 0x9b, 0xa0, 0x2e, 0xa5, 
  0x24, 0x29, 0x3d, 0x33, 0x38, 0xaa, 0x42, 0x47,
  0x4c, 0x51, 0x56, 0x8c, 0x60, 0x65, 0x87, 0x82, 
  0x5b, 0x6a, 0xb4, 0xb9, 0xbe, 0xe1, 0xe6, 0xeb,
  0xf0, 0xf5, 0xfa, 0x73, 0x78, 0x7d, 0x56, 0x05, 
  0xea, 0x32};

int val_1BE1 = 0;
int g_soupmod = 0;
int g_lower_mask = 0x20;
int g_upper_mask = 0xdf;
int g_soupmod2 = 0;
int g_use_text2buffer = 0;

void set_soup_caps(int on) {
  if (on) {
    g_soupmod = 0;
    g_soupmod2 = 0x80;
    g_lower_mask = 0x0;
    g_upper_mask = 0xff;
  } else {
    g_soupmod = 0x0;
    g_lower_mask = 0x20;
    g_upper_mask = 0xdf;
    g_soupmod2 = 0x00;
  }
}

void handle_soup_letter(int entry) {
  if (entry >= 'A') {
    if ((g_soupmod&0x80) || (g_soupmod2&0x80)==0) {
      entry|= g_lower_mask;
    }
    entry &= g_upper_mask;
  }

  g_upper_mask = 0xff;
  if (entry == '.' ||
      entry == LF ||
      entry == 10 ||
      entry == '>' ||
      entry == ' ' ) {
    g_soupmod2 = 0xff;
    // gah, this is a hack..
    if (entry == LF || g_my_stats.screen == SCR_ENCYCLO_SHIP)
      g_upper_mask = 0xdf;
  } else {
    g_soupmod2 = 0;
  }

  handle_letter(entry&0xff);
  if (entry == LF && g_have_icon) {
    g_yc++;
    g_have_icon = 0;
  }
}


void process_letter(int entry) {
  // $12FE - .ex in elite source
  if (entry < 0x20) {
    // jump table
    if (entry == 1) {
      g_lower_mask = 0x0;
    }
    else if (entry == 2) {
      g_lower_mask = 0x20;
      g_soupmod = 0;
    }
    else if (entry == 3) {
      //name planet
      //textmod = 0x80;
      text2buffer(entry);
    }
    else if (entry == 4) {
      //gbaprintf("entry 4!\n");
      if (g_my_stats.screen == SCR_ENCYCLO) {
        textmod = 0x80;
        g_use_text2buffer = 0xff;
      }
      else {
        text2buffer(entry);
      }
    }
    else if (entry == 5) {
      //text2buffer(entry);
      // don't use text buffer?
      g_use_text2buffer = 0;
    }
    else if (entry == 6) {
      g_soupmod = 0x80;
      g_lower_mask = 0x20;
      if (g_my_stats.screen != SCR_ENCYCLO) {
        textmod = 0x80;
        g_use_text2buffer = 0xff;
      }
    }
    else if (entry == 8) {
      set_text_x(6);
      set_soup_caps(0);
    }
    else if (entry == 24 || entry == 22/*|| entry == 9*/) {  // not sure why I thought 9 did this? seems to set xc=1
      if (entry == 22 && (g_my_stats.screen == SCR_ENCYCLO)) {
        // should just tab the data across..
        set_text_x(13);
        set_soup_caps(0);
      } else {
        while (!KeyPressedNoBounce(KEY_A,10)) {
          UpdateKeys();
          wait_vsync_int();
        }
        wait_letgo_A();
        clear_text();
        g_yc = 6;
      }
    }
    else if (entry == 12 || entry == 10 ) {
      g_upper_mask = 0xff;
      handle_soup_letter(entry);
    }
    else if (entry == 13) {
      g_soupmod = 0x80;
      g_lower_mask = 0x20;
    }
    else if (entry == 14) {
      g_soupmod = 0x80;
    }
    else if (entry == 15) {
      g_soupmod = 0x0;
    }
    else if (entry == 17) {
      // "Lavian"
      textmod &= 0xbf;
      text2buffer(0x3); // name planet
      char lastc = g_last_letter | 0x20;
      if (lastc == 'a' ||
          lastc == 'e' ||
          lastc == 'i' ||
          lastc == 'o' ||
          lastc == 'u') {
#if USE_VARIABLE_WIDTH_FONT
        // rewind a letter
        vw_rewind_char();
#else
        g_xc--;
        if (g_xc < 0) {
          g_xc = 29;
          g_yc--;
        }
#endif
      }
      g_soupmod2 = 0;
      dictionary2(0x99); // add ian
    } else if (entry == 18) {
      // add some random letters
      textmod &= 0xbf;
      // carry flag always clear here
      int carry = 0;
      int y = get_rand_carry(&carry)&3;
      // carry flag now depends on last addition in get_rand
      do {
        int x = get_rand_carry(&carry)&0x3e;
        char a = QQ16[x+2]; // ABOUSE etc
        handle_soup_letter(a);
        a = QQ16[x+3]; // ABOUSE etc
        // This fixes a bug in Elite. Look at BEESRI's planet info for details. Essena?oid!
        if (a != '?')
          handle_soup_letter(a);
        carry = 1;
      } while (--y >= 0); // after this, carry flag is definitely set
    } 
    else if (entry == 19) {
      //g_soupmod = 0x80;
      //g_lower_mask = 0x20;
      g_upper_mask = 0xdf;
    }
    /*
    else if (entry == 22) {
      //g_soupmod = 0x80;
      //g_lower_mask = 0x20;
      // clear screen?
      gbaprintf("%0x\n", entry);
      set_text_x(12);
      set_soup_caps(0);
    }*/
    else if (entry == 27) {
      unsigned char ** tmpdictionary = g_dictionary;
      g_dictionary = (unsigned char **)soup_source;
      dictionary2(0xd9 + g_commander.galaxy);
      g_dictionary = tmpdictionary;
      
      
    }
    else if (entry == 28) {
      unsigned char ** tmpdictionary = g_dictionary;
      g_dictionary = (unsigned char **)soup_source;
      dictionary2(0xdc + g_commander.galaxy);
      g_dictionary = tmpdictionary;
      
      
    }
  } else {
    /*// check something?
    if (entry  & val_1BE1  & 0x80) {
      text2buffer(entry);
      } else {*/
    if (entry <= 'Z') {
      handle_soup_letter(entry);
    } else if (entry < 0x81) {
      // carry flag always clear here
      int X = get_rand();
      int A = 0;
      A += (X >= 0x33);
      A += (X >= 0x66);
      A += (X >= 0x99);
      A += (X >= 0xcc);
      int new_entry = A + random_table[entry-'Z']; //table is at 5570
      unsigned char ** tmpdictionary = g_dictionary;
      g_dictionary = (unsigned char **)soup_source;
      dictionary2(new_entry&0xff);
      g_dictionary = tmpdictionary;
      return;
    } else if (entry < 0xd7) {
      if (g_my_stats.screen < SCR_ENCYCLO) {
        unsigned char ** tmpdictionary = g_dictionary;
        g_dictionary = (unsigned char **)soup_source;
        dictionary2(entry);
        g_dictionary = tmpdictionary;
      } else {
        g_dictionary = (unsigned char **)soup_source;
        dictionary2(entry);
        g_dictionary = (unsigned char **)encyc_data1;

      } 
    } else {
      int indx = (entry - 0xd7)*2;
      handle_soup_letter(QQ16[indx]&0xff);
      if (QQ16[indx+1] != '?')
        handle_soup_letter(QQ16[indx+1]&0xff);
    }
    //}
  }
}

void dictionary2(int word) {
  // expand word 
  //printf("start dict %x \n ",word);
  // we want word word...
  unsigned char * str = g_dictionary[word];
  while (*str != 0) {
    int entry = *str++;
    if (g_use_text2buffer) {
      // this is only used on the encyclopedia screens
      unsigned char ** tmp = g_dictionary;
      g_dictionary = text_data;
      text2buffer(entry);
      g_dictionary = tmp;
      g_use_text2buffer = 0;
    } else {
      process_letter(entry);
    }
  }
  
  /*
   * 6502 works more like this:
  int di = 0;
  while (word > 0) {
    while (g_dictionary[di]!=0) {
      di++;
    } 
    word--;
    di++;
  }
  while (g_dictionary[di] != 0) {
    int entry = g_dictionary[di++];
    if (g_use_text2buffer) {
      // this is only used on the encyclopedia screens
      unsigned char * tmp = g_dictionary;
      g_dictionary = text_data;
      text2buffer(entry);
      g_dictionary = tmp;
      g_use_text2buffer = 0;
    } else {
      process_letter(entry);
    }
  }
  */
}

void encocyloDictionary(int word) {
  // expand word 
  //printf("start dict %x \n ",word);
  // we want word word...
  unsigned char * str = g_dictionary[word];
  while (*str != 0) {
    int entry = *str++;
    if (g_use_text2buffer) {
      // this is only used on the encyclopedia screens
      unsigned char ** tmp = g_dictionary;
      g_dictionary = text_data;
      text2buffer(entry);
      g_dictionary = tmp;
      g_use_text2buffer = 0;
    } else {
      if (entry == 0xea || entry == 0xf3) {
        // the next letter is an icon...
        // either 8*8 or 16*16
        int icon = *str++;
        if (entry == 0xea) {
          // next byte is colour of this row (for IFF)
          set_row_pal(g_yc,icon);
        }  else { // 0xf3
          // assume the icon is 16*16
          if (icon & 0x80) {
            // use flight tiles
            draw_icon((icon&0x7f), g_xc, g_yc, 0);
          } else {
            // use docked tiles
            draw_icon((icon&0x7f), g_xc, g_yc, 1);
          }
          g_have_icon = 1;
        }
      } else {
        process_letter(entry);
      }
    }
  }
}

void mission2buffer(int A) {
  g_upper_mask = 0xdf;
  g_dictionary = (unsigned char **)mission_data;
  g_use_text2buffer = 0;
  textmod = 0x80;
  dictionary2(A);
  g_dictionary = (unsigned char **)text_data;
}
void encyclo2buffer(int A) {
  g_upper_mask = 0xdf;
  g_dictionary = (unsigned char **)encyc_data1;
  g_use_text2buffer = 0;
  
  textmod = 0x80;
  encocyloDictionary(A);
  // dictionary2(A);
  g_dictionary = (unsigned char **)text_data;
}
void soup2buffer(int A) {

  g_soupmod = 0x0;
  g_lower_mask = 0x20;
  g_upper_mask = 0xdf;
  g_soupmod2 = 0;
  
  // for encyclopedia.. or rather, so we don't use it normally
  g_use_text2buffer = 0;
  
  g_dictionary = (unsigned char **)soup_source;
  dictionary2(A);
  g_dictionary = (unsigned char **)text_data;
}

void goat_soup(seed_t * seed) {
  //$21C9 $95  STA $00,X  ; randomseed.s0,s1,s2,s3 = planet.s2,s3,s4,s5
  g_rand_seed.r0 = seed->s2;
  g_rand_seed.r1 = seed->s3;
  g_rand_seed.r2 = seed->s4;
  g_rand_seed.r3 = seed->s5;
  soup2buffer(5);
}

// old ex method of strings, just in case
#if 1
void custom2buffer(int word) {
  g_dictionary = gba_textdata;
  string2buffer(gba_textdata[word]);
  g_dictionary = text_data;
}

void ex(int word) {
  string2buffer(text_data[word]);
  /*
  unsigned char * str = text_data[word];
  while (*str) {
    int entry = *str++;
    text2buffer(entry);
  }*/
  /*
  int di = 0;
  while (word > 0) {
    while (ex_source[di]!=0) {
      di++;
    } 
    word--;
    di++;
  }*/
  /*
  while (ex_source[di] != 0) {
    int entry = ex_source[di++];
    text2buffer(entry^0x23);
  }*/
}
#endif

