#include "keypad.h"
#include "screenmode.h"
#include "sprite.h"

#include "gbahandler.h"
#include "title.h"
#include "universe.h"
#include "motion.h"
#include "shipdata.h"
#include "gfx.h"
#include "text.h"
#include "soup.h"
#include "eliteloop.h"
#include "commander.h"
#include "loadsave.h"
#include "dialgauge.h"
#include "flying.h"
#include "input.h"
#include "clipping.h"
#include "options.h"
#include "status.h"
#include "intro2.h"

#include "soundfx.h"

#define TITLE_Y_POS 0


void reset() {
  init_bgs();
  zero_objects();
  
  g_my_stats.screen = SCR_LAUNCHED;
  update_dials();
  dial(DIAL_FU, g_commander.fuel/2);
  dial(DIAL_AL, 29);
  al_ct_scanner(0);
  
  set_missile_count(1);
  // set a lot of things to 0
}

// 5100 n elite source f
void title(int type, int stringid) {
  wait_vsync_int();
  reset();

  // load_scanner();
  // clear_screen();
  if (stringid == 0) {
    set_text_x(7);
    g_yc = 1;
    text2buffer(ELITE_TITLE);
    set_text_x(4);
    g_yc = 11;
    soup2buffer(LOAD_COMM);
    /*
       set_text_x(6);
       g_yc++;
       soup2buffer(BY_BRABEN_BELL);
    */
    //soup2buffer(MOD_DUGAN);
    CopyOAM();
    screen_fade_in();
  } else {
    set_text_x(4);
    g_yc = 11;
    soup2buffer(SPACE_FIRE);
  }
  set_text_x(8);
  soup2buffer(C_ACORNSOFT);

  player_alpha = 0;
  player_beta = 0;
  g_beta = 0;
  g_delta = 0;
  g_alpha = 3;
  int index = add_new_ship(type);
  // set ship rotmat to initial value
  g_objects[index].rotmat[2].z = ROTMAT_1;
  g_objects[index].location.z = 0x4000;
  g_objects[index].location.x = 0;
  g_objects[index].location.y = TITLE_Y_POS;

  g_objects[index].rotx = ROTATION_CLOCKWISE;
  g_objects[index].rotz = ROTATION_CLOCKWISE;

  // now enter title loop
  game_frames = 0;
  g_mcount= 0;
  for (;;) {
    g_mcount++;
    g_mcount&=0xff;
    UpdateKeys();
#ifdef ALLOW_RENDER_CHANGE
    if (KeyPressedNoBounce(KEY_SELECT,3) ) {
      g_RENDER_AS_TRIANGLES ^= 1;
    }
#endif
    wait_vsync_int();
    if (KeyPressed(KEY_START) && stringid == 0) {
      g_need_combat_seq = 0;
      break;
    }
    if (KeyPressed(KEY_B) || KeyPressed(KEY_START)) {
      break;
    }
    if (KeyPressed(KEY_A)) {
      if (stringid == 0) {
        // load screen
        extern signed char g_key_repeat[EVK_COUNT];
        g_key_repeat[EVK_A] = 5;
        sxmp_pause(1);
        restore_game();
        if (!g_game_options.music_off)
          sxmp_pause(0);
      }
      break;
    }

    // make sure that the frame rate is constant
    if (game_frames < g_game_options.frame_rate)
      continue;
    game_frames = 0;

    swap_buffer();
    clear_screen();
    if( g_objects[index].location.z > 0x220) {
      g_objects[index].location.z -= 0x100;
    }
    mveit(&g_objects[index], ships[type-1],index);
    g_objects[index].location.z &= ~0xff;
    g_objects[index].location.z |= 0x80;
    g_objects[index].location.x = 0;
    g_objects[index].location.y = TITLE_Y_POS;
    entry(&g_objects[index], ships[type-1]);
  }
}

void start(void) {

#ifdef HAVE_GBA_SHIPSETS
  load_shipset();
  ships[SHIP_COBRA_MK3-1] = ship_collections[SHIP_COBRA_MK3-1]->ships[4];
  ships[SHIP_KRAIT-1] = ship_collections[SHIP_KRAIT-1]->ships[3];
#else
  ships = ship_collections[13];
  current_ship_flags = ship_flags[13];
#endif
  g_my_stats.game_over = default_stats.game_over;
  set_default_options();
  // reset the shields etc
  reset_stats();
  restore_commander(&tmp_save_commander);

  sxmp_fake_stereo(1);
  play_song(SONG_ELITE_THEME);
  title(SHIP_COBRA_MK3,0);
  int cont = 0;
  while (KeyPressed(KEY_B) || KeyPressed(KEY_A) || KeyPressed(KEY_START)) {
    UpdateKeys();
    wait_vsync_int();
    cont++;
    if (cont > 60)
      break;
  }

  if (g_need_combat_seq) {
    flying_logo();
  
  } else if ( !(g_my_stats.combat_seq==0 && !g_my_stats.docked)) {
    title(SHIP_KRAIT+(g_is_gba-1),1);
    screen_fade_out();
    cont = 0;
    while (KeyPressed(KEY_B) || KeyPressed(KEY_A) || KeyPressed(KEY_START)) {
      UpdateKeys();
      wait_vsync_int();
      cont++;
      if (cont > 60)
        break;
    }
  }
}
