#include "sprite.h"
#include "trumbles.h"
#include "soundfx.h"
#include "soup.h"
#include "commander.h"
#include "gfx.h"

#define FOOD_CARGO 0
#define NARCOTICS_CARGO 6

#define BREED_RATIO   256
#define BREED_CHANCE  0xdc
void load_trumble_data();
void create_trumble(int id,int pal);

typedef struct {
  signed short x;
  signed short y;
  //unsigned char y;
  char  colour;
  short  dx,dy;
  char  active;
}trumble_t;
static trumble_t trumble_positions[8]  __attribute__ ((section (".ewram"))) = {
  {-1,-1, 0,0,0,0},
  {-1,-1, 0,0,0,0},
  {-1,-1, 0,0,0,0},
  {-1,-1, 0,0,0,0},
  {-1,-1, 0,0,0,0},
  {-1,-1, 0,0,0,0},
  {-1,-1, 0,0,0,0},
  {-1,-1, 0,0,0,0}
};

void increment_trumbles() 
{
  // taken from c64 version at $838F
  if (g_commander.trumbles) 
  {
    // they eat food and narcotics
    g_commander.cargo[FOOD_CARGO]=0;
    g_commander.cargo[NARCOTICS_CARGO]=0;
    /*
      ; at this point, A = random
    $839F  AND #$0F   ; A &= 0xf
    $83A1  ADC $04C9  ; A = trumble low + (random&f)
    $83A4  ORA #$04   ; A |= 4
    $83A6  ROL ACC    ; A << 1
    $83A7  STA $04C9  ; trumble low =  ( ( tl+ r&f ) | 4) << 1
    $83AA  ROL $04CA  ; trumble hi << 1 (with carry from low value)
    $83AD  BPL $83B2  ; if positive, jump
    $83AF  ROR $04CA  ; else make sure we don't go negative (max trumbles = 0x7fff)
    */
    // the above is almost ((trumbles + r&f | 4))* 2, but not quite.
    int tmp = (g_commander.trumbles + (get_rand()&0xf))|4;
    tmp *= 2;

    // if low byte +rand was > 0xff then in ROtate Left low bit is also set
    // this is approx the same thing, for near as makes a difference...
    if (tmp & 0x100) {
      tmp|=1;
    }
    if (tmp &0x8000) {
      int low = tmp&0xff;
      tmp = ((tmp&0xff00)/2)|low;
    }
  }
}

void breed_trumbles() 
{
  // this is called every frame!
  if (g_commander.trumbles < BREED_RATIO)
    return;

  if (get_rand() > BREED_CHANCE) 
  {
    g_commander.trumbles++;
    if (g_commander.trumbles == 0x8000)
      g_commander.trumbles--;
  }
  // make a noise
  int r = get_rand();
  int chance = g_commander.trumbles/256;
  if (g_my_stats.cabin_temp < 0xe0)
  {
    chance *= 2;
  }
  if (r < chance) {
    // make some noise... this is tricky
    // if the temp > 0xe0, then a higher pitched squeak is played
    // I have a non-const entry in the sound table, into this I plough
    // some random values...
    extern soundentry_t trumble_squeak[];
    trumble_squeak[0].freq = 0x8000;
    if (get_rand()&1) {
      trumble_squeak[0].freq |= 0x0ff;
      trumble_squeak[2].freq |= 0x0ff;
    } else {
      trumble_squeak[0].freq |= 0x0ff;
      trumble_squeak[2].freq |= 0x0ff;
    }
    
    play_sound(SOUNDFX_TRUMBLE);
  }
}

void generate_new_trumble(int i) {
  // create a new trumble
  trumble_positions[i].colour = 1+(get_rand()&7);
  trumble_positions[i].x = get_rand()&255;
  trumble_positions[i].y = (get_rand()&7)-30;

  create_trumble(i, trumble_positions[i].colour);
  trumble_positions[i].active = 1;
}

int g_trumble_chance = 0;
void view_trumbles() 
{
  // this is called whenever we should draw the laser sprite
  // it sets the flag for the extra trumbles to be shown
  // i.e. call from look1
  if (g_commander.trumbles == 0)
    return;
  // $B000
  g_trumble_chance = g_commander.trumbles/(256*16);
  load_trumble_data();
  int i;
  for (i = 0; i < 8; i++) {
    if (!trumble_positions[i].active && g_trumble_chance > i) {
      generate_new_trumble(i);
    }
    if (trumble_positions[i].active) {
      sprites[TRUMBLE_SPRITE+i].attribute0 &= ~SIZE_DOUBLE; 
      MoveSprite(&sprites[TRUMBLE_SPRITE+i],
          trumble_positions[i].x&0x1ff,
          trumble_positions[i].y&0xff);
    }
  }
}

static const short move_rand[] = {0,1,-1};
void trumble_motion(int frame) 
{
  // call this every frame
  // updates the motion of all trumbles on screen
  if (g_commander.trumbles == 0 ||  g_my_stats.screen)
    return;
  int id = frame&7;
  if (g_trumble_chance > id) {
    // add sprite frame&7..
    // more trumbles == more sprites
    // once a trumble's on screen, it stays there, randomly moving itself
    // note that changing views does not effect the trumbles, they stay where they are
    if (trumble_positions[id].active)
    {
      //gbaprintf("Trumble %d @ %d %d\n", id, trumble_positions[id].x, trumble_positions[id].y);
      // update position
      int r = get_rand();
      if (r > 0xeb) {
        trumble_positions[id].dx = move_rand[r&3];
        trumble_positions[id].dy = move_rand[get_rand()&3];
      }
      
      trumble_positions[id].x += trumble_positions[id].dx;
      trumble_positions[id].y += trumble_positions[id].dy;

      if (trumble_positions[id].y < -30) {
        trumble_positions[id].y = 160;
      } else if (trumble_positions[id].y > 160) {
        trumble_positions[id].y = -30;
      }

      if (trumble_positions[id].x < -30) {
        trumble_positions[id].x = 240;
      } else if (trumble_positions[id].x > 240) {
        trumble_positions[id].x = -30;
      }
      
      MoveSprite(&sprites[TRUMBLE_SPRITE+id],
          trumble_positions[id].x&0x1ff,
          trumble_positions[id].y&0xff);
    }
    else 
    {
      generate_new_trumble(id);
    }
  } else {
      trumble_positions[id].active = 0;
  }
}

void hide_trumbles()
{
  int i;
  for (i = 0; i < 8; i++)
  {
    trumble_positions[i].active = 0;
  }
  deactivate_sprites(TRUMBLE_SPRITE,TRUMBLE_SPRITE+8);
}

void fry_trumbles() 
{
  // called every time we pass the sun and temp > 0xf0
  // fries a load of trumbles, halves quantity
  if (g_commander.trumbles == 0)
    return;
  // remove trumble sprites, reduce amount...
  int i;
  for (i = 0; i < 8; i++) 
  {
    sprites[TRUMBLE_SPRITE+i].attribute0 = SIZE_DOUBLE; 
    trumble_positions[i].active = 0;
  }
  g_commander.trumbles /= 2;
}

