#include "screenmode.h"
#include "sxmp_private.h"
// can't create this on the stack if using sound_update from irq
// as it overflows the irq stack size (which is smallish)
u16 tmp_buffer[BUFFER_SIZE];


#define USE_MIX_LOOP 0
#if USE_MIX_LOOP
void mix_loop(int, u16 *, sound_channel_t *) IN_IWRAM;
void down_sample(int, u16*, u8*) IN_IWRAM;
#endif

IN_IWRAM void sxmp_sound_mix(int samples_to_mix) 
{
  //int initial = BGPaletteMem[0];
#if USE_PROFILING
    BGPaletteMem[0] = RGB16(0,31,31);
#endif
  int ch;
  int i;
  {
    u32 * src = (u32*)tmp_buffer;
    int limit = 1 + (samples_to_mix/16);
    for (i = 0; i < limit; i++)
    {
      *src++ = 0;
      *src++ = 0;
      *src++ = 0;
      *src++ = 0;
      *src++ = 0;
      *src++ = 0;
      *src++ = 0;
      *src++ = 0;
    }
  }
#if USE_PROFILING
  BGPaletteMem[0] = RGB16(12,12,12);
#endif

  // mix together the channels
  // each channel is mixed individually into the temp buffer
  
  // mix the left channels
  for (ch = 0; ch < CHANNEL_COUNT/2; ch++) {
    sound_channel_t * channel = &g_channel[ch];

    // data == 0 means that the channel is off
    if (channel->data) {

      // slight optimisation - if we are going to overflow the buffer, then check each iteration
      // otherwise can do a straight copy with no checking
      if ( (channel->position + (channel->increment*samples_to_mix) ) > channel->length)
      {
#if USE_PROFILING
        BGPaletteMem[0] = RGB16(31,0,0);
#endif
        for (i = 0; i < samples_to_mix; i++)
        {
          // copy from rom to our buffer, check for wrap
          tmp_buffer[i] += (channel->data[channel->position>>12]*channel->volume);
          channel->position += channel->increment;
          if (channel->position >= channel->length) {
            if (channel->loop_length) 
            {
              // sound should loop
              while(channel->position >= channel->length) {
                channel->position -= channel->loop_length;
              }

            } else {
              // end the loop
              channel->position = 0;
              channel->data = 0;
              break;
            }
          }
        }
#if USE_PROFILING
        BGPaletteMem[0] = RGB16(12,12,12);
#endif
      } else {
#if USE_PROFILING
        BGPaletteMem[0] = RGB16(0,31,0);
#endif
#if USE_MIX_LOOP
        mix_loop(samples_to_mix, tmp_buffer, channel);
#else
        for (i = 0; i < samples_to_mix; i++)
        {
          // copy from rom to our buffer
          // for mixing this requires addition and division by number of channels
          //g_sound_vars.current_mix_buffer[i] = (channel->data[channel->position>>12]*channel->volume)>>6; 
          tmp_buffer[i] += (channel->data[channel->position>>12]*channel->volume);
          channel->position += channel->increment;
        }
#endif
#if USE_PROFILING
        BGPaletteMem[0] = RGB16(12,12,12);
#endif
      }
    }
  }
#if USE_PROFILING
  BGPaletteMem[0] = RGB16(31,31,31);
#endif
  // shift down from tmp_buffer (contains channels all added together + volume)
#if USE_MIX_LOOP
  down_sample(samples_to_mix, tmp_buffer, g_sound_vars.current_mix_buffer);
#else
  for (i = 0; i < samples_to_mix; i++) 
  {
#if USE_MIX_LOOP
    g_sound_vars.current_mix_buffer[i] = tmp_buffer[i]/(64);
#else
    g_sound_vars.current_mix_buffer[i] = tmp_buffer[i]/(64*CHANNEL_COUNT/2);
#endif
    tmp_buffer[i] = 0;
  }
#endif
#if USE_PROFILING
  BGPaletteMem[0] = RGB16(12,12,12);
#endif

  // mix the right channels
  for (ch = CHANNEL_COUNT/2; ch < CHANNEL_COUNT; ch++) {
    sound_channel_t * channel = &g_channel[ch];

    // data == 0 means that the channel is off
    if (channel->data) {

      // slight optimisation - if we are going to overflow the buffer, then check each iteration
      // otherwise can do a straight copy with no checking
      if ( (channel->position + (channel->increment*samples_to_mix) ) > channel->length)
      {
        
#if USE_PROFILING
        BGPaletteMem[0] = RGB16(31,0,0);
#endif
        for (i = 0; i < samples_to_mix; i++)
        {
          // copy from rom to our buffer, check for wrap
          tmp_buffer[i] += (channel->data[channel->position>>12]*channel->volume);
          channel->position += channel->increment;
          if (channel->position >= channel->length) {
            if (channel->loop_length) 
            {
              // sound should loop
              while(channel->position >= channel->length) {
                channel->position -= channel->loop_length;
              }

            } else {
              // end the loop
              channel->position = 0;
              channel->data = 0;
              break;
            }
          }
        }
#if USE_PROFILING
        BGPaletteMem[0] = RGB16(12,12,12);
#endif
      } else {
#if USE_PROFILING
        BGPaletteMem[0] = RGB16(0,31,0);
#endif
#if USE_MIX_LOOP
        mix_loop(samples_to_mix, tmp_buffer, channel);
#else
        for (i = 0; i < samples_to_mix; i++)
        {
          // copy from rom to our buffer
          // for mixing this requires addition and division by number of channels
          //g_sound_vars.current_mix_buffer[i] = (channel->data[channel->position>>12]*channel->volume)>>6; 
          tmp_buffer[i] += (channel->data[channel->position>>12]*channel->volume);
          channel->position += channel->increment;
        }
#endif
#if USE_PROFILING
        BGPaletteMem[0] = RGB16(12,12,12);
#endif
      }
    }
  }
#if USE_PROFILING
  BGPaletteMem[0] = RGB16(31,31,31);
#endif
#if USE_MIX_LOOP
  down_sample(samples_to_mix, tmp_buffer, g_sound_vars.current_mix_buffer_2);
#else
  for (i = 0; i < samples_to_mix; i++) 
  {
#if USE_MIX_LOOP
    g_sound_vars.current_mix_buffer_2[i] = tmp_buffer[i]/(64);
#else
    g_sound_vars.current_mix_buffer_2[i] = tmp_buffer[i]/(64*CHANNEL_COUNT/2);
#endif
  }
#endif
  // increment counter
  g_sound_vars.current_mix_buffer += samples_to_mix;
  g_sound_vars.current_mix_buffer_2 += samples_to_mix;
#if USE_PROFILING
  BGPaletteMem[0] = RGB16(0,0,12);
#endif
  
}

#if 1
// call this when no samples need mixing
IN_IWRAM void sxmp_no_channel_mix(int samples_to_mix) 
{
  int i;
  if (samples_to_mix&3) {
  for (i = 0; i < samples_to_mix; i++) {
    g_sound_vars.current_mix_buffer[i] = g_sound_vars.current_mix_buffer_2[i] = 0;
  }
  } else {
    u32 * b1 = (u32*)g_sound_vars.current_mix_buffer;
    u32 * b2 = (u32*)g_sound_vars.current_mix_buffer_2;
    for (i = 0; i < samples_to_mix/4; i++) {
      *b1++ = 0;
      *b2++ = 0;
    }
  }
  g_sound_vars.current_mix_buffer += samples_to_mix;
  g_sound_vars.current_mix_buffer_2 += samples_to_mix;
}
#endif
